@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 exception Escape of exn
@@ -23,5 +23,5 @@ val run: string -> unit
 val print_exn: Format.formatter -> exn -> unit
 
 
-val eval: string -> (Ns.qname option * Value.t) list
+val eval: string -> (Atoms.V.t option * Value.t) list
   (* Can be used from CDuce units *)
