@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 (* Decompilation of regular expressions *)
@@ -22,7 +22,15 @@ module type S = sig
   val hash: t -> int
 end
 
-module Decompile(X : Hashtbl.S)(S : S)
+module type TABLE = sig
+  type key
+  type 'a t
+  val create: int -> 'a t
+  val add: 'a t -> key -> 'a -> unit
+  val find: 'a t -> key -> 'a
+end
+
+module Decompile(X : TABLE)(S : S)
 : sig
   val decompile: (X.key -> (S.t * X.key) list * bool) -> X.key -> S.t regexp
 end
