@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 #load "q_MLast.cmo";;
@@ -28,12 +28,13 @@ let vars = ref [||]
 let memo_typ = IntHash.create 13
 
 let atom lab = Types.atom (Atoms.atom (Atoms.V.mk_ascii lab))
-let label lab = LabelPool.mk (Ns.empty, U.mk lab)
+let label lab = Label.mk (Ns.empty, U.mk lab)
 let bigcup f l = List.fold_left (fun accu x -> Types.cup accu (f x)) Types.empty l
 
 let rec typ t =
   try IntHash.find memo_typ t.uid
   with Not_found ->
+(*    print_int t.uid; print_char ' '; flush stdout; *)
     let node = Types.make () in
     IntHash.add memo_typ t.uid node;
     Types.define node (typ_descr t.def);
@@ -47,7 +48,7 @@ and typ_descr = function
   | Variant (_,l,_) -> bigcup variant l
   | Record (_,l,_) ->
       let l = List.map (fun (lab,t) -> label lab, typ t) l in
-      Types.record' (false,(LabelMap.from_list_disj l))
+      Types.record_fields (false,(LabelMap.from_list_disj l))
   | Abstract "int" -> Builtin_defs.caml_int
   | Abstract "char" -> Builtin_defs.char_latin1
   | Abstract "string" -> Builtin_defs.string_latin1
@@ -58,6 +59,7 @@ and typ_descr = function
   | Builtin ("Big_int.big_int", []) -> Builtin_defs.int
   | Builtin ("Cduce_lib.Value.t", []) -> Types.any
   | Builtin ("Cduce_lib.Encodings.Utf8.t", []) -> Builtin_defs.string
+  | Builtin ("Cduce_lib.Atoms.V.t", []) -> Builtin_defs.atom
   | Builtin ("unit", []) -> Sequence.nil_type
   | Builtin ("option", [t]) -> Sequence.option (typ t)
   | Var i -> Types.descr (!vars).(i)
@@ -81,7 +83,7 @@ let mk_var _ =
 
 let mk_vars = List.map mk_var
 
-let loc = (Lexing.dummy_pos,Lexing.dummy_pos)
+let _loc = (Lexing.dummy_pos,Lexing.dummy_pos)
 
 let let_in p e body =
   <:expr< let $list:[ p, e ]$ in $body$ >>
@@ -121,6 +123,8 @@ let protect e f =
 (* Registered types *)
 
 let gen_types = ref true
+(* currently always off *)
+
 
 module HashTypes = Hashtbl.Make(Types)
 let registered_types = HashTypes.create 13
@@ -195,8 +199,8 @@ let abstr_lab l x res =
 
 
 let rec to_cd e t =
-(*  Format.fprintf Format.std_formatter "to_cd %a [uid=%i; recurs=%i]@."
-    Mltypes.print t t.uid t.recurs; *)
+(*  Format.fprintf Format.err_formatter "to_cd %a [uid=%i; recurs=%i]@."
+    Mltypes.print t t.uid t.recurs;  *)
   if t.recurs > 0 then <:expr< $lid:to_cd_fun t$ $e$ >>
   else to_cd_descr e t.def
 
@@ -301,6 +305,8 @@ and to_cd_descr e = function
   | Builtin ("Cduce_lib.Value.t", []) -> e
   | Builtin ("Cduce_lib.Encodings.Utf8.t", []) -> 
       <:expr< Value.ocaml2cduce_string_utf8 $e$ >>
+  | Builtin ("Cduce_lib.Atoms.V.t", []) ->
+      <:expr< Value.ocaml2cduce_atom $e$ >>
   | Builtin ("unit", []) -> <:expr< do { $e$; Value.nil } >>
   | Var _ -> e
   | Builtin ("option", [t]) ->
@@ -315,8 +321,8 @@ and tuple_to_cd tl vars = List.map2 (fun t id -> to_cd <:expr< $lid:id$ >> t) tl
 
 
 and to_ml e t =
-(*  Format.fprintf Format.std_formatter "to_ml %a@."
-    Mltypes.print t; *)
+(*  Format.fprintf Format.err_formatter "to_ml %a@."
+    Mltypes.print t;  *)
   if t.recurs > 0 then <:expr< $lid:to_ml_fun t$ $e$ >>
   else to_ml_descr e t.def
 
@@ -348,7 +354,6 @@ and to_ml_descr e = function
 	 | "B",Some x -> `B (t(x))
 	 | _ -> assert false
       *)
-      let x = mk_var () in
       let cases = 
 	List.map 
 	  (function 
@@ -429,6 +434,8 @@ and to_ml_descr e = function
   | Builtin ("Cduce_lib.Value.t", []) -> e
   | Builtin ("Cduce_lib.Encodings.Utf8.t", []) -> 
       <:expr< Value.cduce2ocaml_string_utf8 $e$ >>
+  | Builtin ("Cduce_lib.Atoms.V.t", []) ->
+      <:expr< Value.cduce2ocaml_atom $e$ >>
   | Builtin ("unit", []) -> <:expr< ignore $e$ >>
   | Builtin ("option", [t]) ->
       <:expr< Value.cduce2ocaml_option $lid:to_ml_fun t$ $e$ >>
@@ -468,7 +475,7 @@ let exts = ref []
 
 let check_value ty_env c_env (s,caml_t,t) =
   (* Find the type for the value in the CDuce module *)
-  let id = Id.mk (Ns.empty, U.mk s) in
+  let id = (Ns.empty, U.mk s) in
   let vt = 
     try Typer.find_value id ty_env
     with Not_found ->
@@ -476,7 +483,6 @@ let check_value ty_env c_env (s,caml_t,t) =
       "The interface exports a value %s which is not available in the module@." s;
       exit 1
   in
-
   (* Compute expected CDuce type *)
   let et = Types.descr (typ t) in
 
@@ -497,32 +503,20 @@ let check_value ty_env c_env (s,caml_t,t) =
     );
    
   (* Generate stub code *)
-  (* let x = t(Eval.get_slot cu slot) *)
   let x = mk_var () in
   let slot = Compile.find_slot id c_env in
-  let e = to_ml <:expr< Eval.get_slot cu $int:string_of_int slot$ >> t in
+  let e = to_ml <:expr< slots.($int:string_of_int slot$) >> t in
   <:patt< $uid:s$ >>, <:expr< C.$uid:x$ >>, (<:patt< $uid:x$ >>, e)
 
-let stub name ty_env c_env values =
-  gen_types := true;
+let stub ty_env c_env exts values mk prolog =
+  gen_types := false;
   let items = List.map (check_value ty_env c_env) values in
 
-  let exts = List.rev_map (fun (s,t) -> to_cd <:expr< $lid:s$ >> t) !exts in
+  let exts = List.rev_map (fun (s,t) -> to_cd <:expr< $lid:s$ >> t) exts in
   let g = global_transl () in
 
-  (* 
-     let (v1,v2,...,vn) = 
-     let module C = struct
-      let cu = ...
-      open Cduce_lib
-      Config.init_all ()
-      let types = ...
-      let rec <global translation functions>
-      <fills external slots>
-      <run the unit>
-      let <stubs for values>
-     end in (C.x1,...,C.xn)
-  *)
+  let types = get_registered_types () in
+  let raw = mk types in
 
   let items_def = List.map (fun (_,_,d) -> d) items in
   let items_expr = List.map (fun (_,e,_) -> e)  items in
@@ -531,27 +525,48 @@ let stub name ty_env c_env values =
   let m = 
     [ <:str_item< open Cduce_lib >>;
       <:str_item< Config.init_all () >>;
-      <:str_item< value types = Librarian.registered_types cu >> ] @
-    (if g = [] then [] else [ <:str_item< value rec $list:g$ >> ]) @    [ <:str_item< Librarian.set_externals cu [|$list:exts$|] >>;
-    <:str_item< Librarian.run cu >> ] @
-    (if items = [] then [] else [ <:str_item< value $list:items_def$ >> ]) in
+      <:str_item< 
+	value (types,set_externals,slots,run) = 
+	Librarian.ocaml_stub $str:String.escaped raw$ >> ] @
+      (if g = [] then [] else [ <:str_item< value rec $list:g$ >> ]) @    
+      [ <:str_item< set_externals [|$list:exts$|] >>;
+	<:str_item< run () >> ] @
+      (if items = [] then [] else [ <:str_item< value $list:items_def$ >> ]) in
 
   let items_expr = 
     match items_expr with 
       | [] -> <:expr< () >> 
       | l -> <:expr< ($list:l$) >> in
 
-  <:patt< ($list:items_pat$) >>, m, items_expr
+  let str_items = 
+    [ <:str_item< 
+      value ($list:items_pat$) = 
+	let module C = struct 
+	  $list:m$ 
+	end in $items_expr$ >>, (Lexing.dummy_pos, Lexing.dummy_pos) ] in
 
-let stub_ml cu ty_env c_env = 
+  print_endline prolog;
+  !Pcaml.print_implem str_items
+(*  let exe = Filename.concat (Filename.dirname Sys.argv.(0)) "cdo2ml" in
+  let oc = Unix.open_process_out exe in
+  Marshal.to_channel oc str_items [];
+  flush oc;
+  ignore (Unix.close_process_out oc) *)
+
+
+let stub_ml name ty_env c_env exts mk =
   try
-    let name = String.capitalize cu in
+    let name = String.capitalize name in
+    let exts = match (Obj.magic exts : (string * Mltypes.t) list option) with
+      | None -> []
+      | Some exts -> List.iter (fun (_,t) -> Mltypes.reg_uid t) exts; exts in
+    (* First, read the description of ML types for externals.
+       Don't forget to call reg_uid to avoid uid clashes...
+       Do that before reading the cmi. *)
     let (prolog, values) = 
       try Mltypes.read_cmi name
       with Not_found ->  ("",[]) in
-    let code = stub cu ty_env c_env values in
-    Some (Obj.magic (prolog,code)),
-    get_registered_types ()
+    stub ty_env c_env exts values mk prolog
   with Mltypes.Error s -> raise (Location.Generic s)
 
 
@@ -604,7 +619,7 @@ let gen_wrapper vals =
 	 try (s,fst (Mltypes.find_value s)) :: accu
 	 with Not_found ->
 	   let vals = 
-	     try Mltypes.load_cmi s
+	     try Mltypes.load_module s
 	     with Not_found ->
 	       failwith ("Cannot resolve " ^ s)
 	   in
@@ -614,6 +629,25 @@ let gen_wrapper vals =
     wrapper values
   with Mltypes.Error s -> raise (Location.Generic s)
 
+let make_wrapper fn = 
+  let ic = open_in fn in
+  let v = ref [] in
+  (try while true do 
+     let s = input_line ic in
+     if s <> "" then
+       match s.[0] with
+	 | 'A'..'Z' -> v := s :: !v 
+	 | '#' -> ()
+	 | _ -> failwith "Error in primitive file: names must start with a capitalized letter"
+   done 
+   with End_of_file -> ());
+  let s = gen_wrapper !v in
+  !Pcaml.print_implem [ s,_loc ];
+  print_endline "let () = Librarian.obj_path := [";
+  List.iter (fun s -> Printf.printf "  %S;\n" s) !Librarian.obj_path;
+  print_endline " ];;";
+  print_endline "let () = Run.main ();;"
+
 
 (* Dynamic coercions *)
 
@@ -645,7 +679,9 @@ let register () =
   Typer.has_ocaml_unit :=
     (fun cu -> Mltypes.has_cmi (U.get_str cu));
   Librarian.stub_ml := stub_ml;
-  Externals.register := register
+  Externals.register := register;
+  Externals.ext_info := (fun () -> Obj.magic !exts);
+  Librarian.make_wrapper := make_wrapper
 
 let () =
   Config.register 
