@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Caml_cduce
@@ -23,9 +23,11 @@ and def =
   | Var of int
 
 
+val reg_uid: t -> unit
+
 (* Load an external .cmi *)
 val has_cmi: string -> bool
-val load_cmi: string -> (string * t) list
+val load_module: string -> (string * t) list
 
 (* Load the .cmi corresponding to a CDuce compilation unit *)
 val read_cmi: string -> string * (string * Types.type_expr * t) list
