@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 #load "pa_extend.cmo";;
@@ -123,8 +123,8 @@ let logical_not e = if_then_else e cst_false cst_true
 let apply_op2_noloc op e1 e2 = Apply (Apply (Var (ident op), e1), e2)
 let apply_op2 loc op e1 e2 = exp loc (apply_op2_noloc op e1 e2)
  
-let set_ref e1 e2 = Apply (Dot (e1, U.mk "set", []), e2)
-let get_ref e = Apply (Dot (e, U.mk "get", []), cst_nil)
+let set_ref e1 e2 = Apply (Dot (e1, U.mk "set"), e2)
+let get_ref e = Apply (Dot (e, U.mk "get"), cst_nil)
 let let_in e1 p e2 =  Match (e1, [p,e2])
 let seq e1 e2 = let_in e1 pat_nil e2
 let concat e1 e2 = apply_op2_noloc "@" e1 e2
@@ -142,42 +142,45 @@ EXTEND
 
   phrase: [
     [ (f,p,e) = let_binding -> 
-	if f then [ mk loc (FunDecl e) ] else
-	  [ mk loc (LetDecl (p,e)) ]
+	if f then [ mk _loc (FunDecl e) ] else
+	  [ mk _loc (LetDecl (p,e)) ]
     | (_,p,e1) = let_binding; "in"; e2 = expr LEVEL "top"->
-        [ mk loc (EvalStatement (exp loc (let_in e1 p e2))) ]
-    | "type"; x = located_ident; "="; t = pat -> [ mk loc (TypeDecl (x,t)) ]
+        [ mk _loc (EvalStatement (exp _loc (let_in e1 p e2))) ]
+    | "type"; x = located_ident; "="; t = pat -> [ mk _loc (TypeDecl (x,t)) ]
     | "using"; name = IDENT; "="; cu = [ IDENT | STRING2 ] ->
-	[ mk loc (Using (U.mk name, U.mk cu)) ]
+	[ mk _loc (Using (U.mk name, U.mk cu)) ]
+    | "open"; ids = LIST1 [ IDENT | keyword ] SEP "." -> 	  
+	let ids = List.map (fun x -> ident x) ids in
+	[ mk _loc (Open ids) ]
     | "schema"; name = IDENT; "="; uri = STRING2 ->
 	protect_op "schema";
-        [ mk loc (SchemaDecl (U.mk name, uri)) ]
+        [ mk _loc (SchemaDecl (U.mk name, uri)) ]
     | n = namespace_binding ->
 	let d = match n with
 	    | `Prefix (name,ns) ->  Namespace (name, ns)
 	    | `Keep b -> KeepNs b in
-	[ mk loc d ]
+	[ mk _loc d ]
     | n = namespace_binding; "in"; e2 = expr LEVEL "top" ->
 	let e = 
 	  match n with
 	    | `Prefix (name,ns) -> NamespaceIn (name, ns, e2)
 	    | `Keep b -> KeepNsIn (b,e2)
 	in
-	[ mk loc (EvalStatement (exp loc e)) ]
-    | "debug"; d = debug_directive -> [ mk loc (Directive (`Debug d)) ]
-    | "#"; IDENT "verbose" -> [ mk loc (Directive `Verbose) ]
-    | "#"; IDENT "silent" -> [ mk loc (Directive `Silent) ]
+	[ mk _loc (EvalStatement (exp _loc e)) ]
+    | "debug"; d = debug_directive -> [ mk _loc (Directive (`Debug d)) ]
+    | "#"; IDENT "verbose" -> [ mk _loc (Directive `Verbose) ]
+    | "#"; IDENT "silent" -> [ mk _loc (Directive `Silent) ]
     | "#"; IDENT "utf8" -> Ulexer.enc := Ulexing.Utf8; [ ]
     | "#"; IDENT "latin1" -> Ulexer.enc := Ulexing.Latin1; [ ]
     | "#"; IDENT "ascii" -> Ulexer.enc := Ulexing.Ascii; [ ]
-    | "#"; IDENT "quit" -> [ mk loc (Directive `Quit) ]
-    | "#"; IDENT "env" -> [ mk loc (Directive `Env) ]
+    | "#"; IDENT "quit" -> [ mk _loc (Directive `Quit) ]
+    | "#"; IDENT "env" -> [ mk _loc (Directive `Env) ]
     | "#"; IDENT "print_type"; t = pat ->
-        [ mk loc (Directive (`Print_type t)) ]
-    | "#"; IDENT "dump_value"; e = expr -> [ mk loc (Directive (`Dump e)) ]
-    | "#"; IDENT "reinit_ns" -> [ mk loc (Directive `Reinit_ns) ]
-    | "#"; IDENT "help" -> [ mk loc (Directive `Help) ]
-    | "#"; IDENT "builtins" -> [ mk loc (Directive `Builtins) ]
+        [ mk _loc (Directive (`Print_type t)) ]
+    | "#"; IDENT "dump_value"; e = expr -> [ mk _loc (Directive (`Dump e)) ]
+    | "#"; IDENT "reinit_ns" -> [ mk _loc (Directive `Reinit_ns) ]
+    | "#"; IDENT "help" -> [ mk _loc (Directive `Help) ]
+    | "#"; IDENT "builtins" -> [ mk _loc (Directive `Builtins) ]
     | "include"; s = STRING2 ->
 	let s = 
 	  if Filename.is_relative s 
@@ -207,7 +210,7 @@ EXTEND
 	       include_stack := List.tl !include_stack)
 	)
     ] | 
-    [ e = expr -> [ mk loc (EvalStatement e) ]
+    [ e = expr -> [ mk _loc (EvalStatement e) ]
     ]
   ];
 
@@ -217,7 +220,6 @@ EXTEND
     | IDENT "compile"; t = pat; p = LIST1 pat -> `Compile (t,p)
     | IDENT "sample"; t = pat -> `Sample t
     | IDENT "subtype"; t1 = pat; t2 = pat -> `Subtype (t1,t2)
-    | IDENT "explain"; t0 = pat; t = pat; e = expr -> `Explain (t0,t,e)
     | IDENT "single"; t = pat -> `Single t
     ]
   ];
@@ -238,43 +240,43 @@ EXTEND
   expr: [
     "top" RIGHTA
     [ "match"; e = SELF; "with"; b = branches -> 
-	exp loc (Match (e,b))
+	exp _loc (Match (e,b))
     | "try"; e = SELF; "with"; b = branches -> 
-	exp loc (Try (e,b))
+	exp _loc (Try (e,b))
     | "map"; e = SELF; "with"; b = branches -> 
-	exp loc (Map (e,b))
+	exp _loc (Map (e,b))
     | "xtransform"; e = SELF; "with"; b = branches -> 
-	exp loc (Xtrans (e,b))
+	exp _loc (Xtrans (e,b))
     | "if"; e = SELF; "then"; e1 = SELF; "else"; e2 = SELF ->
-	exp loc (if_then_else e e1 e2)
+	exp _loc (if_then_else e e1 e2)
     | "transform"; e = SELF; "with"; b = branches -> 
-	exp loc (Transform (e,b))
+	exp _loc (Transform (e,b))
     | "validate"; e = SELF; "with"; (schema, typ) = schema_ref ->
-        exp loc (Validate (e, schema, typ))
+        exp _loc (Validate (e, [schema;typ]))
     | "select"; e = SELF; "from"; 
       l = LIST1 [ x = pat ; "in"; e = expr -> (x,e)] SEP "," ;
       cond = [ "where"; c = LIST1 [ expr ] SEP "and" -> c 
-	     | -> [] ] -> exp loc (SelectFW (e,l,cond)) 
+	     | -> [] ] -> exp _loc (SelectFW (e,l,cond)) 
     | "fun"; (f,a,b) = fun_decl ->
-	exp loc (Abstraction { fun_name = f; fun_iface = a; fun_body = b })
+	exp _loc (Abstraction { fun_name = f; fun_iface = a; fun_body = b })
     | (_,p,e1) = let_binding; "in"; e2 = expr LEVEL "top"->
-        exp loc (let_in e1 p e2)
+        exp _loc (let_in e1 p e2)
     | n = namespace_binding; "in"; e2 = expr LEVEL "top" ->
 	(match n with
-	   | `Prefix (name,ns) -> exp loc (NamespaceIn (name, ns, e2))
-	   | `Keep f -> exp loc (KeepNsIn (f,e2)))
+	   | `Prefix (name,ns) -> exp _loc (NamespaceIn (name, ns, e2))
+	   | `Keep f -> exp _loc (KeepNsIn (f,e2)))
     | e = expr; ":"; p = pat ->
-	exp loc (Forget (e,p))
+	exp _loc (Forget (e,p))
     | e = expr; ":"; "?"; p = pat ->
-	exp loc (Check (e,p))
+	exp _loc (Check (e,p))
     | e1 = expr; ";"; e2 = expr ->
-	exp loc (seq e1 e2) 
+	exp _loc (seq e1 e2) 
     | "ref"; p = pat; e = expr ->
-	exp loc (Ref (e,p))
-    | "not"; e = expr -> exp loc (logical_not e)
+	exp _loc (Ref (e,p))
+    | "not"; e = expr -> exp _loc (logical_not e)
     ]
     |
-    [ e1 = expr; ":="; e2 = expr -> exp loc (set_ref e1 e2)
+    [ e1 = expr; ":="; e2 = expr -> exp _loc (set_ref e1 e2)
     ]
     | 
     [ e1 = expr; op = ["=" | "<=" | "<<" | ">>" | ">=" ]; e2 = expr -> 
@@ -282,38 +284,38 @@ EXTEND
 	  | "<<" -> "<"
 	  | ">>" -> ">"
 	  | s -> s in
-	apply_op2 loc op e1 e2
+	apply_op2 _loc op e1 e2
     ]
 
     | 
-    [ e1 = expr; op = ["+" | "-" | "@" ]; e2 = expr -> apply_op2 loc op e1 e2
-    | e1 = expr; ["||" | "or"]; e2 = expr -> exp loc (logical_or e1 e2)
+    [ e1 = expr; op = ["+" | "-" | "@" ]; e2 = expr -> apply_op2 _loc op e1 e2
+    | e1 = expr; ["||" | "or"]; e2 = expr -> exp _loc (logical_or e1 e2)
     | e = expr; "\\"; l = [IDENT | keyword ] -> 
-	exp loc (RemoveField (e, label l)) 
+	exp _loc (RemoveField (e, label l)) 
     ]
     |
-    [ e1 = expr; op = ["*"]; e2 = expr -> apply_op2 loc op e1 e2
-    | e1 = expr; "&&"; e2 = expr -> exp loc (logical_and e1 e2)
+    [ e1 = expr; op = ["*"]; e2 = expr -> apply_op2 _loc op e1 e2
+    | e1 = expr; "&&"; e2 = expr -> exp _loc (logical_and e1 e2)
     | e = expr; op = "/"; p = pat LEVEL "simple" ->
 	(* transform e with <(Atom)>[($$$::t|_)*] -> [$$$] *)
-	let tag = mk loc (Internal (Types.atom (Atoms.any))) in
-	let att = mk loc (Internal Types.Record.any) in
-	let any = mk loc (Internal Types.any) in
+	let tag = mk _loc (Internal (Types.atom (Atoms.any))) in
+	let att = mk _loc (Internal Types.Record.any) in
+	let any = mk _loc (Internal Types.any) in
 	let re = Star(Alt(SeqCapture(noloc,id_dummy,Elem p), Elem any)) in
-	let ct = mk loc (Regexp re) in
-        let p = mk loc (XmlT (tag, multi_prod loc [att;ct])) in
-	exp loc (Transform (e,[p, Var id_dummy]))
+	let ct = mk _loc (Regexp re) in
+        let p = mk _loc (XmlT (tag, multi_prod _loc [att;ct])) in
+	exp _loc (Transform (e,[p, Var id_dummy]))
     | e = expr; "/@"; a = [IDENT|keyword] ->
 	(* transform e with <(Atom) {a=$$$}>_ -> [$$$] *)
-        let tag = mk loc (Internal (Types.atom Atoms.any)) in
-        let any = mk loc (Internal Types.any) in
-        let att = mk loc (Record
+        let tag = mk _loc (Internal (Types.atom Atoms.any)) in
+        let any = mk _loc (Internal Types.any) in
+        let att = mk _loc (Record
 			    (true, [(label a,
-				     (mk loc (PatVar (None,id_dummy)),
+				     (mk _loc (PatVar [id_dummy]),
 				      None))])) in
-        let p = mk loc (XmlT (tag, multi_prod loc [att;any])) in
+        let p = mk _loc (XmlT (tag, multi_prod _loc [att;any])) in
         let t = (p, Pair (Var id_dummy,cst_nil)) in
-        exp loc (Transform (e,[t]))
+        exp _loc (Transform (e,[t]))
      | e = expr; "//" ; p = pat ->
 	 (*
 	   let $stack=ref [p*] [] in
@@ -325,32 +327,34 @@ EXTEND
 	   set_ref 
 	     (Var stk)
 	     (concat (get_ref (Var stk)) (Pair (Var id_dummy,cst_nil))) in
-	 let capt = mk loc (And (mk loc (PatVar (None,U.mk "$$$")),p)) in
+	 let capt = mk _loc (And (mk _loc (PatVar [U.mk "$$$"]),p)) in
 	 let xt = Xtrans (e,[capt,assign]) in
-	 let rf = Ref (cst_nil, mk loc (Regexp (Star(Elem p)))) in
+	 let rf = Ref (cst_nil, mk _loc (Regexp (Star(Elem p)))) in
 	 let body = 
-	   let_in rf (mk loc (PatVar (None,stk)))
-	     (let_in xt (mk loc (Internal Types.any)) (get_ref (Var stk)))
+	   let_in rf (mk _loc (PatVar [stk]))
+	     (let_in xt (mk _loc (Internal Types.any)) (get_ref (Var stk)))
 	 in
-	 exp loc body
+	 exp _loc body
     ]
     | [ 
-      e1 = SELF; IDENT "div"; e2 = expr -> apply_op2 loc "/" e1 e2
-    | e1 = SELF; IDENT "mod"; e2 = expr -> apply_op2 loc "mod" e1 e2
-    | e1 = SELF; e2 = expr -> exp loc (Apply (e1,e2))
+      e1 = SELF; IDENT "div"; e2 = expr -> apply_op2 _loc "/" e1 e2
+    | e1 = SELF; IDENT "mod"; e2 = expr -> apply_op2 _loc "mod" e1 e2
+    | e1 = SELF; e2 = expr -> exp _loc (Apply (e1,e2))
     ]
 
     | "no_appl" 
 
-    [ e = expr;  "."; l = [IDENT | keyword ];
-      tyargs = [ "with"; "{"; pl = LIST0 pat; "}" -> pl | -> [] ]
-      -> 
-	exp loc (Dot (e, label l,tyargs)) 
+    [ e = expr;  "."; l = [IDENT | keyword ]; 
+      tyargs = [ "with"; "{"; tyargs = LIST0 pat; "}" -> Some tyargs
+      | -> None ] ->
+	let e = Dot (e,label l) in
+	match tyargs with None -> exp _loc e | Some tyargs -> 
+	  exp _loc (TyArgs (e,tyargs))
     ]  
     | [ 
-      "("; l = LIST1 expr SEP ","; ")" -> exp loc (tuple l)
+      "("; l = LIST1 expr SEP ","; ")" -> exp _loc (tuple l)
     | "[";  l = LIST0 seq_elem; e = OPT [ ";"; e = expr -> e ]; 
-      loc_end = ["]" -> loc] ->
+      loc_end = ["]" -> _loc] ->
 	let e = match e with Some e -> e | None -> cst_nil in
         let e = exp loc_end e in
         let (_,loc_end) = loc_end in
@@ -362,39 +366,39 @@ EXTEND
 		       | `Explode x -> concat x q
 		  ) l e
 	in
-	exp loc l
+	exp _loc l
     | "<"; t = [ "("; e = expr; ")" -> e
-	       | a = tag -> exp loc a
+	       | a = tag -> exp _loc a
 	       ];
 	a = expr_attrib_spec; ">"; c = expr ->
-	  exp loc (Xml (t, Pair (a,c)))
+	  exp _loc (Xml (t, Pair (a,c)))
     | "{"; r = expr_record_spec; "}" -> r
     | s = STRING2 ->
 	let s = U.mk s in
-	exp loc (String (U.start_index s, U.end_index s, s, cst_nil))
-    | a = IDENT -> exp loc (Var (ident a))
-    | "!"; e = expr -> exp loc (get_ref e)
-    | i = INT -> exp loc (Integer (Intervals.V.mk i))
+	exp _loc (String (U.start_index s, U.end_index s, s, cst_nil))
+    | a = IDENT -> exp _loc (Var (ident a))
+    | "!"; e = expr -> exp _loc (get_ref e)
+    | i = INT -> exp _loc (Integer (Intervals.V.mk i))
     | "`"; a = tag -> a
-    | c = char -> exp loc (Char c)
+    | c = char -> exp _loc (Char c)
     ]
 
   ];
 
-  tag: [ [ a = [ IDENT | keyword ] -> exp loc (Atom (ident a)) ] ];
+  tag: [ [ a = [ IDENT | keyword ] -> exp _loc (Atom (ident a)) ] ];
 
   tag_type: [
-    [ IDENT "_" ->  mk loc (Internal (Types.atom Atoms.any))
-    | a = [ IDENT | keyword ] -> mk loc (Cst (Atom (ident a)))
-    | t = ANY_IN_NS -> mk loc (NsT (ident t)) 
+    [ "_" ->  mk _loc (Internal (Types.atom Atoms.any))
+    | a = [ IDENT | keyword ] -> mk _loc (Cst (Atom (ident a)))
+    | t = ANY_IN_NS -> mk _loc (NsT (ident t)) 
     ]
   ];
 
   seq_elem: [
     [ x = STRING1 -> 
 	let s = U.mk x in
-	`String (loc, U.start_index s, U.end_index s, s)
-    | e = expr LEVEL "no_appl" -> `Elems (loc,e)
+	`String (_loc, U.start_index s, U.end_index s, s)
+    | e = expr LEVEL "no_appl" -> `Elems (_loc,e)
     | "!"; e = expr LEVEL "no_appl" -> `Explode e
     ]
   ];
@@ -404,21 +408,26 @@ EXTEND
 	[ name = 
 	    [ name = [ IDENT | keyword ]; "=" -> ident name
 	    | -> U.mk "" ];
-	  uri = STRING2 ->
-	    let ns = Ns.mk (ident uri) in
-	    `Prefix (name,ns)
+	  ns = ns_expr -> `Prefix (name,ns)
 	| IDENT "on" -> `Keep true
 	| IDENT "off" -> `Keep false ]
       ] -> r ]
   ];
 
+  ns_expr: [
+    [ uri = STRING2 -> `Uri (Ns.Uri.mk (ident uri))
+    | ids = LIST1 [ IDENT | keyword ] SEP "." -> 	  
+	let ids = List.map (fun x -> ident x) ids in
+	`Path ids ]
+  ];
+
   
   let_binding: [
     [ "let"; is_fun_decl; OPT "fun"; (f,a,b) = fun_decl ->
 	let f = match f with Some x -> x | None -> assert false in
-	let p = mk loc (PatVar (None, snd f)) in
+	let p = mk _loc (PatVar [snd f]) in
 	let abst = { fun_name = Some f; fun_iface = a; fun_body = b } in
-        let e = exp loc (Abstraction abst) in
+        let e = exp _loc (Abstraction abst) in
         (true,p,e)
     | "let"; p = pat; "="; e = expr -> (false,p,e)
     | "let"; p = pat; ":"; t = pat; "="; e = expr -> (false,p, Forget (e,t))
@@ -493,16 +502,16 @@ EXTEND
   regexp: [ 
     [ x = regexp; "|"; y = regexp -> 
 	match (x,y) with
-	  | Elem x, Elem y -> Elem (mk loc (Or (x,y)))
+	  | Elem x, Elem y -> Elem (mk _loc (Or (x,y)))
 	  | _ -> Alt (x,y) 
     ]
   | [ x = regexp; y = regexp -> Seq (x,y) ]
   | [ x = regexp; "&"; y = regexp ->
 	match (x,y) with
-	  | Elem x, Elem y -> Elem (mk loc (And (x,y)))
-	  | _ -> error loc "Conjunction not allowed in regular expression"
+	  | Elem x, Elem y -> Elem (mk _loc (And (x,y)))
+	  | _ -> error _loc "Conjunction not allowed in regular expression"
     ]
-  | [ a = IDENT; "::"; x = regexp -> SeqCapture (lop loc,ident a,x) ] 
+  | [ a = IDENT; "::"; x = regexp -> SeqCapture (lop _loc,ident a,x) ] 
   | [ x = regexp; "*" -> Star x
     | x = regexp; "*?" -> WeakStar x
     | x = regexp; "+" -> Seq (x, Star x)
@@ -519,10 +528,10 @@ EXTEND
 	    let i = int_of_string i in
 	    if (i > 1024) then raise Exit else i
 	      (* We cannot handle type that huge... *)
-	  with Failure _ | Exit -> error loc "Repetition number too large"
+	  with Failure _ | Exit -> error _loc "Repetition number too large"
 	in
 	if (i <= 0) then 
-	  error loc "Repetition number must be a positive integer";
+	  error _loc "Repetition number must be a positive integer";
 	aux i Epsilon
     ]
   | [ "("; x = LIST1 regexp SEP ","; ")" ->
@@ -533,18 +542,18 @@ EXTEND
 		List.map 
 		  (function
 		     | Elem x -> x
-		     | _ -> error loc 
+		     | _ -> error _loc 
 			 "Mixing regular expressions and products")
 		  x in
-	      Elem (multi_prod loc x))
+	      Elem (multi_prod _loc x))
     | "("; a = IDENT; ":="; c = expr; ")" -> 
-	Elem (mk loc (Constant ((ident a,c))))
+	Elem (mk _loc (Constant ((ident a,c))))
     | "/"; p = pat LEVEL "simple" -> Guard p
     | IDENT "PCDATA" -> string_regexp
     | i = STRING1; "--"; j = STRING1 ->
-	let i = Chars.V.mk_int (parse_char loc i)
-	and j = Chars.V.mk_int (parse_char loc j) in
-        Elem (mk loc (Internal (Types.char (Chars.char_class i j))))
+	let i = Chars.V.mk_int (parse_char _loc i)
+	and j = Chars.V.mk_int (parse_char _loc j) in
+        Elem (mk _loc (Internal (Types.char (Chars.char_class i j))))
     |  s = STRING1 ->
 	List.fold_right
 	  (fun c accu -> 
@@ -562,69 +571,70 @@ EXTEND
     ]
   ];
 
-  located_ident: [ [ a = [IDENT|keyword] -> (lop loc,ident a) ] ];
+  located_ident: [ [ a = [IDENT|keyword] -> (lop _loc,ident a) ] ];
 
   pat: [ 
       [ x = pat; "where"; 
         b = LIST1 [ (la,a) = located_ident; "="; y = pat -> 
 		      (la,a,y) ] SEP "and"
-            -> mk loc (Recurs (x,b)) ]
-    | RIGHTA [ x = pat; "->"; y = pat -> mk loc (Arrow (x,y))
-             | x = pat; "@"; y = pat -> mk loc (Concat (x,y))
-             | x = pat; "+"; y = pat -> mk loc (Merge (x,y)) ]
-    | "no_arrow" [ x = pat; "|"; y = pat -> mk loc (Or (x,y)) ] 
-    | "simple" [ x = pat; "&"; y = pat -> mk loc (And (x,y)) 
-	       | x = pat; "\\"; y = pat -> mk loc (Diff (x,y)) ]
+            -> mk _loc (Recurs (x,b)) ]
+    | RIGHTA [ x = pat; "->"; y = pat -> mk _loc (Arrow (x,y))
+             | x = pat; "@"; y = pat -> mk _loc (Concat (x,y))
+             | x = pat; "+"; y = pat -> mk _loc (Merge (x,y)) ]
+    | "no_arrow" [ x = pat; "|"; y = pat -> mk _loc (Or (x,y)) ] 
+    | "simple" [ x = pat; "&"; y = pat -> mk _loc (And (x,y)) 
+	       | x = pat; "\\"; y = pat -> mk _loc (Diff (x,y)) ]
     | 
       [ "{"; r = record_spec; "}" -> r
       | "ref"; p = pat ->
-	  let get_fun = mk loc (Arrow (pat_nil, p)) 
-	  and set_fun = mk loc (Arrow (p, pat_nil))in
+	  let get_fun = mk _loc (Arrow (pat_nil, p)) 
+	  and set_fun = mk _loc (Arrow (p, pat_nil))in
 	  let fields = 
 	    [ label "get", (get_fun, None); label "set", (set_fun, None) ] in
-	  mk loc (Record (false, fields))
-      | IDENT "_" -> mk loc (Internal Types.any)
+	  mk _loc (Record (false, fields))
+      | "_" -> mk _loc (Internal Types.any)
       | "("; a = IDENT; ":="; c = expr; ")" -> 
-	  mk loc (Constant (ident a,c))
+	  mk _loc (Constant (ident a,c))
       | "!"; a = IDENT ->
-	  mk loc (Internal (Types.abstract (Types.Abstract.atom a)))
-      | cu = OPT [ cu = IDENT; "." -> U.mk cu ]; a = [ IDENT | keyword ] ->
-	  mk loc (PatVar (cu, ident a))
+	  mk _loc (Internal (Types.abstract (Types.Abstract.atom a)))
+      | ids = LIST1 [ IDENT | keyword ] SEP "." ->
+	  let ids = List.map (fun x -> ident x) ids in
+	  mk _loc (PatVar ids)
       | i = INT ; "--"; j = INT -> 
           let i =  Intervals.V.mk i 
 	  and j =  Intervals.V.mk j in
-          mk loc (Internal (Types.interval (Intervals.bounded i j)))
+          mk _loc (Internal (Types.interval (Intervals.bounded i j)))
       | i = INT -> 
           let i =  Intervals.V.mk i  in
-          mk loc (Internal (Types.interval (Intervals.atom i)))
+          mk _loc (Internal (Types.interval (Intervals.atom i)))
       | "*"; "--"; j = INT ->
 	  let j =  Intervals.V.mk j in
-          mk loc (Internal (Types.interval (Intervals.left j)))
+          mk _loc (Internal (Types.interval (Intervals.left j)))
       | i = INT; "--"; "*" ->
 	  let i = Intervals.V.mk i in
-          mk loc (Internal (Types.interval (Intervals.right i)))
+          mk _loc (Internal (Types.interval (Intervals.right i)))
       | i = char ->
-          mk loc (Internal (Types.char (Chars.char_class i i)))
+          mk _loc (Internal (Types.char (Chars.char_class i i)))
       | i = char ; "--"; j = char ->
-          mk loc (Internal (Types.char (Chars.char_class i j)))
+          mk _loc (Internal (Types.char (Chars.char_class i j)))
       | "`"; c = tag_type -> c
-      | "("; l = LIST1 pat SEP ","; ")" -> multi_prod loc l
+      | "("; l = LIST1 pat SEP ","; ")" -> multi_prod _loc l
       | "["; r = [ r = regexp -> r | -> Epsilon ];
              q = [ ";"; q = pat -> Some q
                  | -> None ]; 
              "]" -> 
 	       let r = match q with
 		 | Some q -> 	
-		     let any = mk loc (Internal (Types.any)) in
+		     let any = mk _loc (Internal (Types.any)) in
 		     Seq(r,Seq(Guard q, Star (Elem any)))
 		 | None -> r
 	       in
-	       mk loc (Regexp r)
+	       mk _loc (Regexp r)
       | "<"; t =
             [ x = tag_type -> x
             | "("; t = pat; ")" -> t ];
 	a = attrib_spec; ">"; c = pat ->
-          mk loc (XmlT (t, multi_prod loc [a;c]))
+          mk _loc (XmlT (t, multi_prod _loc [a;c]))
       | s = STRING2 ->
 	  let s = 
 	    List.map 
@@ -635,7 +645,7 @@ EXTEND
 				   (Chars.V.mk_int c))))) 
 	      (seq_of_string s) in
 	  let s = s @ [mknoloc (Internal (Sequence.nil_type))] in
-	  multi_prod loc s
+	  multi_prod _loc s
       ]
     
   ];
@@ -650,18 +660,18 @@ EXTEND
     [ [ r = LIST0 [ l = [IDENT | keyword ]; f = opt_field_pat; OPT ";" ->
 		      let (o,x,y) =
 			match f with
-			  | None -> (false, mknoloc (PatVar (None,ident l)), None)
+			  | None -> (false, mknoloc (PatVar [ident l]), None)
 			  | Some z -> z
 		      in
-		      let x = if o then mk loc (Optional x) else x in
+		      let x = if o then mk _loc (Optional x) else x in
 		      (label l, (x,y))
                   ]; op = [ ".." -> true | -> false ] ->
-	  mk loc (Record (op,r))
+	  mk _loc (Record (op,r))
       ] ];
   
   char:
     [ 
-      [ c = STRING1 -> Chars.V.mk_int (parse_char loc c) ]
+      [ c = STRING1 -> Chars.V.mk_int (parse_char _loc c) ]
     ];
      
 
@@ -679,7 +689,7 @@ EXTEND
 		  let x = match x with Some x -> x | None ->  Var (ident l) in
 		  (label l,x) ] 
 	  ->
-	  exp loc (RecordLitt r)
+	  exp _loc (RecordLitt r)
       ] ];
 
   expr_attrib_spec: [
