@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Pxp_yacc
@@ -64,7 +64,7 @@ let load_pxp handlers uri =
     process_entity pxp_config 
       (`Entry_document[`Extend_dtd_fully]) mgr handlers;
   with exn ->
-    raise (Location.Generic (Pxp_types.string_of_exn exn))
+    Value.failwith' (Pxp_types.string_of_exn exn)
 
 let use () = Load_xml.xml_parser := load_pxp pxp_handle_event
 
