@@ -1,10 +1,8 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
-open Value
-
-val run_dispatcher: Patterns.Compile.dispatcher -> t -> int * t array
+val run_dispatcher: Auto_pat.state -> Value.t -> int * Value.t array
 
