@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Ident
@@ -11,12 +11,11 @@ type t =
   | Pair of t * t
   | Xml of t * t * t
   | XmlNs of t * t * t * Ns.table
-  | Record of t label_map
+  | Record of t Imap.t
   | Atom of Atoms.V.t
   | Integer of Intervals.V.t
   | Char of Chars.V.t
   | Abstraction of (Types.descr * Types.descr) list option * (t -> t)
-  | Abstraction2 of t array * (Types.t * Types.t) list * Lambda.branches
   | Abstract of Types.Abstract.V.t
   | String_latin1 of int * int * string * t
   | String_utf8 of Utf8.uindex * Utf8.uindex * Utf8.t * t
@@ -47,11 +46,11 @@ let vfalse = Atom (Atoms.V.mk_ascii "false")
 let vbool x = if x then vtrue else vfalse
 
 let vrecord l =
-  let l = List.map (fun (qname,v) -> LabelPool.mk qname, v) l in
-  Record (LabelMap.from_list_disj l)
+  let l = List.map (fun (lab,v) -> Upool.int lab, v) l in
+  Record (Imap.create (Array.of_list l))
 
 let get_fields = function
-  | Record map -> LabelMap.mapi_to_list (fun k v -> LabelPool.value k, v) map
+  | Record map -> Obj.magic (Imap.elements map)
   | _ -> raise (Invalid_argument "Value.get_fields")
 
 let rec sequence = function
@@ -64,6 +63,19 @@ let rec sequence_rev accu = function
 
 let sequence_rev l = sequence_rev nil l
 
+let sequence_of_array a =
+  let rec aux accu i =
+    if (i = 0) then accu
+    else let i = pred i in aux (Pair (a.(i), accu)) i in
+  aux nil (Array.length a)
+
+let tuple_of_array a =
+  let rec aux accu i =
+    if (i = 0) then accu
+    else let i = pred i in aux (Pair (a.(i), accu)) i in
+  let n = Array.length a in
+  aux a.(n) (pred n)
+
 let concat v1 v2 = 
   match (v1,v2) with
     | (Atom _, v) | (v, Atom _) -> v
@@ -83,14 +95,19 @@ let rec const = function
   | Types.Pair (x,y) -> Pair (const x, const y)
   | Types.Xml (x, Types.Pair (y, z)) -> Xml (const x, const y, const z)
   | Types.Xml (_,_) -> assert false
-  | Types.Record x -> Record (LabelMap.map const x)
+  | Types.Record x -> 
+      let x = LabelMap.mapi_to_list (fun l c -> (Upool.int l,const c)) x in
+      Record (Imap.create (Array.of_list x))
   | Types.String (i,j,s,c) -> String_utf8 (i,j,s, const c)
 
 let rec inv_const = function
   | Pair (x, y) -> Types.Pair (inv_const x, inv_const y)
   | Xml (x, y, z) | XmlNs (x,y,z,_) ->
       Types.Pair (inv_const x, Types.Pair (inv_const y, inv_const z))
-  | Record x -> Types.Record (LabelMap.map inv_const x)
+  | Record x -> 
+      let x = Imap.elements x in
+      let x = List.map (fun (l,c) -> (Label.from_int l,inv_const c)) x in
+      Types.Record (LabelMap.from_list_disj x)
   | Atom a -> Types.Atom a
   | Integer i -> Types.Integer i
   | Char c -> Types.Char c
@@ -259,12 +276,11 @@ let rec print ppf v =
   else  match v with
     | Pair (x,y) -> Format.fprintf ppf "(%a,%a)" print x print y
     | Xml (x,y,z) | XmlNs (x,y,z,_) -> print_xml ppf x y z
-    | Record l -> Format.fprintf ppf "{%a }" print_record (LabelMap.get l)
+    | Record l -> Format.fprintf ppf "{%a }" print_record (Imap.elements l)
     | Atom a ->	Atoms.V.print_quote ppf a
     | Integer i -> Intervals.V.print ppf i
     | Char c ->	Chars.V.print ppf c
     | Abstraction _ -> Format.fprintf ppf "<fun>"
-    | Abstraction2 _ -> Format.fprintf ppf "<fun>"
     | String_latin1 (i,j,s,q) -> 
 	Format.fprintf ppf "<string_latin1:%i-%i,%S,%a>" i j s print q
     | String_utf8 (i,j,s,q) -> 
@@ -326,7 +342,7 @@ and print_tag ppf = function
   | Atom tag -> Atoms.V.print ppf tag
   | tag -> Format.fprintf ppf "(%a)" print tag
 and print_attr ppf = function
-  | Record attr -> print_record ppf (LabelMap.get attr)
+  | Record attr -> print_record ppf (Imap.elements attr)
   | attr -> Format.fprintf ppf "(%a)" print attr
 
 and print_record ppf = function
@@ -334,7 +350,7 @@ and print_record ppf = function
   | f :: rem -> Format.fprintf ppf " %a" print_field f; print_record ppf rem
 
 and print_field ppf (l,v) = 
-  Format.fprintf ppf "%a=%a" Label.print (LabelPool.value l) print v  
+  Format.fprintf ppf "%a=%a" Label.print_attr (Label.from_int l) print v  
 
 let dump_xml ppf v =
   let rec aux ppf = function
@@ -347,7 +363,7 @@ let dump_xml ppf v =
     | Record x ->
         Format.fprintf ppf "@[<hv1>";
         Format.fprintf ppf "<record>@,%a@,</record>@]"
-          (fun ppf x -> print_record ppf (LabelMap.get x)) x
+          (fun ppf x -> print_record ppf (Imap.elements x)) x
     | Atom a ->
         Format.fprintf ppf "@[<hv1>";
         Format.fprintf ppf "<atom>@,%a@,</atom>@]"
@@ -363,9 +379,6 @@ let dump_xml ppf v =
     | Abstraction _ ->
         Format.fprintf ppf "@[<hv1>";
         Format.fprintf ppf "<abstraction />@]"
-    | Abstraction2 _ ->
-        Format.fprintf ppf "@[<hv1>";
-        Format.fprintf ppf "<abstraction2 />@]"
     | Abstract (s,_) ->
 	Format.fprintf ppf "<abstract>%s</abstract>" s
     | String_latin1 (_, _, s, v) ->
@@ -401,18 +414,17 @@ let rec compare x y =
 	  let c = compare x1 y1 in if c <> 0 then c 
 	  else let c = compare x2 y2 in if c <> 0 then c 
 	  else compare x3 y3
-      | Record rx, Record ry -> LabelMap.compare compare rx ry
+      | Record rx, Record ry -> Imap.compare compare rx ry
       | Atom x, Atom y -> Atoms.V.compare x y
       | Integer x, Integer y -> Intervals.V.compare x y
       | Char x, Char y -> Chars.V.compare x y
       | Abstraction (_,_), _
-      | _, Abstraction (_,_)
-      | Abstraction2 (_,_,_), _
-      | _, Abstraction2 (_,_,_) -> 
+      | _, Abstraction (_,_) ->
 	  raise (CDuceExn (string_latin1 "comparing functional values"))
       | Abstract (s1,v1), Abstract (s2,v2) ->
 	  let c = Types.Abstract.T.compare s1 s2 in if c <> 0 then c 
-	  else raise (CDuceExn (string_latin1 "comparing abstract values"))
+	  else Pervasives.compare (Obj.magic v1 : float) (Obj.magic v2 : float)
+	    (* raise (CDuceExn (string_latin1 "comparing abstract values")) *)
       | Absent,_ | _,Absent ->
 	  Format.fprintf Format.std_formatter
 	    "ERR: Compare %a %a@." print x print y;
@@ -466,6 +478,26 @@ let rec compare x y =
       | Integer _,_ -> -1 | _, Integer _ -> 1
       | Abstract _, _ -> -1 | _, Abstract _ -> 1
 
+let rec hash = function
+  | Pair (x1,x2) ->
+      1 + hash x1 * 257 + hash x2 * 17 
+  | (Xml (x1,x2,x3) | XmlNs (x1,x2,x3,_)) ->
+      2 + hash x1 * 65537 + hash x2 * 257 + hash x3 * 17
+  | Record rx -> 
+      3 + 17 * Imap.hash hash rx
+  | Atom x -> 
+      4 + 17 * Atoms.V.hash x
+  | Integer x -> 
+      5 + 17 * Intervals.V.hash x
+  | Char x -> 
+      6 + 17 * Chars.V.hash x
+  | Abstraction _ -> 7
+  | Abstract _ -> 8
+  | Absent -> assert false
+  | Concat _ as x -> eval_lazy_concat x; hash x
+  | String_latin1 (i,j,s,q) -> hash (normalize_string_latin1 i j s q)
+  | String_utf8 (i,j,s,q) ->hash (normalize_string_utf8 i j s q)
+
 let iter_xml pcdata_callback other_callback =
   let rec aux = function
     | v when compare v nil = 0 -> ()
@@ -512,7 +544,7 @@ let map_xml map_pcdata map_other =
 
 let tagged_tuple tag vl =
   let ct = sequence vl in
-  let at = Record LabelMap.empty in
+  let at = Record Imap.empty in
   let tag = Atom (Atoms.V.mk_ascii tag) in
   Xml (tag, at, ct)
 
@@ -584,10 +616,10 @@ let get_variant = function
       | _ -> assert false
 
 let label_ascii s =
-  LabelPool.mk (Ns.empty, U.mk s)
+  Label.mk_ascii s
 
-let record l =
-  Record (LabelMap.from_list_disj l)
+let record (l : (label * t) list) =
+  Record (Imap.create (Array.of_list (Obj.magic l)))
 
 let record_ascii l =
   record (List.map (fun (l,v) -> (label_ascii l, v)) l)
@@ -595,7 +627,7 @@ let record_ascii l =
 
 let get_field v l =
   match v with
-    | Record fields -> LabelMap.assoc l fields
+    | Record fields -> Imap.find fields (Upool.int l)
     | _ -> raise Not_found
 
 let get_field_ascii v l = get_field v (label_ascii l)
@@ -608,10 +640,10 @@ let get_abstract = function
   | _ -> assert false
   
 
-let get_label = LabelPool.mk (Ns.empty, U.mk "get") 
-let set_label = LabelPool.mk (Ns.empty, U.mk "set")
+let get_label = Upool.int (label_ascii "get")
+let set_label = Upool.int (label_ascii "set")
 let mk_rf ~get ~set =
-  LabelMap.from_list_disj [ get_label, get; set_label, set ]
+  Imap.create [| get_label, get; set_label, set |]
 
 let mk_ref t v =
   let r = ref v in
@@ -637,13 +669,27 @@ let cduce2ocaml_int = function
   | Integer i -> Intervals.V.get_int i
   | _ -> assert false
 
-let ocaml2cduce_string = string_latin1
+let ocaml2cduce_int32 i =
+  Integer (Intervals.V.from_int32 i)
 
-let cduce2ocaml_string = get_string_latin1
+let cduce2ocaml_int32 = function
+  | Integer i -> Intervals.V.to_int32 i
+  | _ -> assert false
 
-let ocaml2cduce_string_utf8 = string_utf8
+let ocaml2cduce_int64 i =
+  Integer (Intervals.V.from_int64 i)
 
-let cduce2ocaml_string_utf8 s = fst (get_string_utf8 s)
+let cduce2ocaml_int64 = function
+  | Integer i -> Intervals.V.to_int64 i
+  | _ -> assert false
+
+let ocaml2cduce_string s = string_latin1 (String.copy s)
+
+let cduce2ocaml_string = get_string_latin1 (* Result is already fresh *)
+
+let ocaml2cduce_string_utf8 s = string_utf8 (U.mk (String.copy (U.get_str s)))
+
+let cduce2ocaml_string_utf8 s = fst (get_string_utf8 s) (* Result is already fresh *)
 
 let ocaml2cduce_char c =
   Char (Chars.V.mk_char c)
@@ -655,17 +701,15 @@ let cduce2ocaml_char = function
   | Char c -> Chars.V.to_char c 
   | _ -> assert false
 
-(*
 let ocaml2cduce_bigint i =
   Integer (Intervals.V.from_bigint i)
 
 let cduce2ocaml_bigint = function
   | Integer i -> Intervals.V.get_bigint i
   | _ -> assert false
-*)
 
-let ocaml2cduce_atom ns l =
-  Atom (Atoms.V.mk (Ns.mk ns) l)
+let ocaml2cduce_atom q = Atom q
+let cduce2ocaml_atom = function Atom a -> a | _ -> assert false
 
 let print_utf8 v =
   print_string (U.get_str v);
@@ -689,9 +733,30 @@ let ocaml2cduce_option f = function
 
 let add v1 v2 = match (v1,v2) with
   | (Integer x, Integer y) -> Integer (Intervals.V.add x y)
-  | (Record r1, Record r2) -> Record (LabelMap.merge (fun x y -> y) r1 r2)
+  | (Record r1, Record r2) -> Record (Imap.merge r1 r2)
   | _ -> assert false
 
+let merge v1 v2 = match (v1,v2) with
+  | (Record r1, Record r2) -> Record (Imap.merge r1 r2)
+  | _ -> assert false
+
+let sub v1 v2 = match (v1,v2) with
+  | (Integer x, Integer y) -> Integer (Intervals.V.sub x y)
+  | _ -> assert false
+
+let mul v1 v2 = match (v1,v2) with
+  | (Integer x, Integer y) -> Integer (Intervals.V.mult x y)
+  | _ -> assert false
+
+let div v1 v2 = match (v1,v2) with
+  | (Integer x, Integer y) -> Integer (Intervals.V.div x y)
+  | _ -> assert false
+
+let modulo v1 v2 = match (v1,v2) with
+  | (Integer x, Integer y) -> Integer (Intervals.V.modulo x y)
+  | _ -> assert false
+
+
 let pair v1 v2 = Pair (v1,v2)
 let xml v1 v2 v3 = Xml (v1,v2,v3)
 
@@ -699,8 +764,103 @@ let mk_record labels fields =
   let l = ref [] in
   assert (Array.length labels == Array.length fields);
   for i = 0 to Array.length labels - 1 do
-    let (ns,lab) = labels.(i) in
-    l := (LabelPool.mk (Ns.mk ns, lab), fields.(i)) :: !l
+    l := (labels.(i),fields.(i)) :: !l;
   done;
-  Record (LabelMap.from_list_disj !l)
+  record !l
   
+
+(* TODO: optimize cases
+     - (f x = [])
+     - all chars copied or deleted *)
+
+let rec transform_aux f accu = function
+  | Pair (x,y) -> let accu = concat accu (f x) in transform_aux f accu y
+  | Atom _ -> accu
+  | v -> transform_aux f accu (normalize v)
+
+let transform f v = transform_aux f nil v
+
+
+let rec xtransform_aux f accu = function
+  | Pair (x,y) -> 
+      let accu = match f x with
+	| Absent ->
+	    let x = match x with
+	      | Xml (tag, attr, child) -> 
+		  let child = xtransform_aux f nil child in
+		  Xml (tag, attr, child)
+	      | XmlNs (tag, attr, child, ns) ->
+		  let child = xtransform_aux f nil child in
+		  XmlNs (tag, attr, child, ns)
+	      | x -> x in
+	    concat accu (Pair (x,nil))
+	| x -> concat accu x
+      in 
+      xtransform_aux f accu y
+  | Atom _ -> accu
+  | v -> xtransform_aux f accu (normalize v)
+
+let xtransform f v = xtransform_aux f nil v
+
+let remove_field l = function
+  | Record r -> Record (Imap.remove r (Upool.int l))
+  | _ -> assert false
+
+let rec ocaml2cduce_list f = function
+  | [] -> nil
+  | hd::tl -> Pair (f hd, ocaml2cduce_list f tl)
+
+let rec cduce2ocaml_list f v =
+  match normalize v with
+    | Pair (x,y) -> f x :: (cduce2ocaml_list f y)
+    | _ -> []
+
+let ocaml2cduce_array f x = ocaml2cduce_list f (Array.to_list x)
+let cduce2ocaml_array f x = Array.of_list (cduce2ocaml_list f x)
+
+let no_attr = Record Imap.empty
+
+let ocaml2cduce_constr tag va = 
+  Xml (tag, no_attr, sequence_of_array va)
+
+let rec cduce2ocaml_constr m = function
+  | Atom q ->
+      Obj.repr (Atoms.get_map q m)
+  | Xml (Atom q,_,f) | XmlNs (Atom q,_,f,_) ->
+	let tag = Atoms.get_map q m in
+	let x = Obj.repr (Array.of_list (get_sequence f)) in
+	Obj.set_tag x tag;
+	x
+  | _ -> assert false
+
+let rec cduce2ocaml_variant m = function
+  | Atom q ->
+      Obj.repr (Atoms.get_map q m)
+  | Xml (Atom q,_,f) | XmlNs (Atom q,_,f,_) ->
+	let tag = Atoms.get_map q m in
+	let (x,_) = get_pair f in
+	Obj.repr (tag,x)
+  | _ -> assert false
+
+      
+let ocaml2cduce_fun farg fres f =
+  Abstraction (None, fun x -> fres (f (farg x)))
+
+let cduce2ocaml_fun farg fres = function
+  | Abstraction (_, f) -> (fun x -> fres (f (farg x)))
+  | _ -> assert false
+
+let apply f arg = match f with
+  | Abstraction (_,f) -> f arg
+  | _ -> assert false
+
+
+type pools = Ns.Uri.value array * Ns.Label.value array
+
+let extract_all () =
+  Ns.Uri.extract (),
+  Ns.Label.extract ()
+ 
+let intract_all (uri,label) =
+  Ns.Uri.intract uri;
+  Ns.Label.intract label
