@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Printf
@@ -49,7 +49,7 @@ let add_limits s = "^" ^ s ^ "$"
 exception Error of string
 let simple_type_error name = raise (Error name)
 
-let qualify s = (Ns.empty, Encodings.Utf8.mk s)
+let qualify = Ns.Label.mk_ascii
 
   (* regular expressions used to validate built-in types *)
 
@@ -565,8 +565,9 @@ let string_of_time_type fields =
     in
     let (positive, hour, minute) = (ref true, ref zero, ref zero) in
     List.iter
-      (fun ((ns, name), value) ->
-        if ns <> Ns.empty then fail ();
+      (fun (lab, value) ->
+	 let ns,name = Ns.Label.value lab in
+        if ns != Ns.empty then fail ();
         (match Utf8.get_str name with
         | "positive" -> positive :=  (Value.equal value Value.vtrue)
         | "hour" -> hour := parse_int value
@@ -576,8 +577,9 @@ let string_of_time_type fields =
     !positive, !hour, !minute
   in
   let parse_time_kind = function
-    | Value.Atom a ->
-        (match Utf8.get_str (snd (Atoms.V.value a)) with
+    | Value.Atom q ->
+	let _,s = Atoms.V.value q in
+        (match Utf8.get_str s with
         | "duration" -> Duration | "dateTime" -> DateTime | "time" -> Time
         | "date" -> Date | "gYearMonth" -> GYearMonth | "gYear" -> GYear
         | "gMonthDay" -> GMonthDay | "gDay" -> GDay | "gMonth" -> GMonth
@@ -625,9 +627,10 @@ let string_of_time_type fields =
   in
   let v =
     List.fold_left
-      (fun acc ((ns, name), value) ->
-        if ns <> Ns.empty then fail ();
-        (match Utf8.get_str name with
+      (fun acc (lab, value) ->
+	 let ns,local = Ns.Label.value lab in
+         if ns != Ns.empty then fail ();
+        (match Utf8.get_str local with
         | "year" -> { acc with year = Some (parse_int value) }
         | "month" -> { acc with month = Some (parse_int value) }
         | "day" -> { acc with day = Some (parse_int value) }
@@ -686,7 +689,7 @@ let string_of_time_type fields =
 
   (** {2 API} *)
 
-let xsd_any = (xsd,Utf8.mk "anyType")
+let xsd_any = add_xsd_prefix "anyType"
 let is s = QTable.mem builtins s || (Ns.QName.equal s xsd_any)
 let iter f = QTable.iter f builtins
 
