@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Encodings
@@ -19,7 +19,7 @@ val simple_type: t -> simple_type_definition
 val cd_type: t -> Types.t
 val validate: t -> Utf8.t -> Value.t
 
-val string_of_time_type: (Ns.qname * Value.t) list -> Utf8.t
+val string_of_time_type: (Ns.Label.t * Value.t) list -> Utf8.t
 
 val any_simple_type: t
 val string: t
