@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Printf
@@ -206,7 +206,7 @@ let schema_of_uri uri =
     let root = node_of_uri uri in
     let targetNamespace =  
       match _may_attr "targetNamespace" root with
-	| Some ns -> Ns.mk ns
+	| Some ns -> Ns.Uri.mk ns
 	| None -> Ns.empty
     in
     let attributeFormDefault = 
@@ -218,7 +218,7 @@ let schema_of_uri uri =
 
   let may_name n =
     match _may_attr "name" n with
-      | Some local -> Some (targetNamespace,local)
+      | Some local -> Some (targetNamespace, local)
       | None -> None in
   let get_name n = (targetNamespace, _attr "name" n) in
 
@@ -232,7 +232,7 @@ let schema_of_uri uri =
       | Simple st -> st
       | _ -> error "Not a simple type"
 
-  and resolve_elt qname =
+  and resolve_elt (qname : Ns.QName.t) =
     resolve "element" elts elts_elems (parse_elt_decl true) qname
       
   and resolve_att qname =
@@ -335,7 +335,7 @@ let schema_of_uri uri =
 	| _ -> Ns.empty
     in
     let typdef = find_simple_type n in
-    { attr_name = (ns,local);
+    { attr_name = Ns.Label.mk (ns,local);
       attr_typdef = typdef;
       attr_cstr = parse_att_value_constraint typdef n }
       
@@ -487,7 +487,7 @@ let schema_of_uri uri =
     let type_def = find_element_type n in
     let value_constr = parse_elt_value_constraint type_def n in
     let nillable = bool_attr "nillable" n in
-    element (ns,local) type_def value_constr nillable
+    element (Atoms.V.mk (ns,local)) type_def value_constr nillable
 	    
   (* look for a type definition, try "simpleType" child, try "complexType"
    * child, try "type" attribute, return anyType *)
@@ -505,12 +505,11 @@ let schema_of_uri uri =
   and parse_particle n =
     let min, max = parse_min_max n in
     let model mg = particle_model min max mg in
-    let elt e n = particle min max (Elt e) (Atoms.atom (Atoms.V.of_qname n )) 
-      (min = 0) in
+    let elt e n = particle min max (Elt e) (Atoms.atom n) (min = 0) in
     match _tag n with
       | "xsd:element" ->
 	  (match _may_qname_attr "ref" n with
-	     | Some ref -> elt (resolve_elt ref) ref
+	     | Some ref -> elt (resolve_elt ref) (Atoms.V.mk ref)
 	     | None ->
 		 let decl = parse_elt_decl false n in
 		 elt decl decl.elt_name)
@@ -541,7 +540,7 @@ let schema_of_uri uri =
   and parse_wildcard_ns = function
     | ns when Utf8.get_str ns = "##targetNamespace" -> targetNamespace
     | ns when Utf8.get_str ns = "##local" -> Ns.empty
-    | ns -> Ns.mk ns
+    | ns -> Ns.Uri.mk ns
 
 
 
@@ -574,7 +573,7 @@ let schema_of_uri uri =
 
   let check_redef n table kind =
     let name = get_name n in
-    if (QTable.mem elts name) then
+    if (QTable.mem table name) then
       error ("Redefinition of " ^ kind ^ " " ^ 
 			  Ns.QName.to_string name)
     else name
@@ -582,11 +581,11 @@ let schema_of_uri uri =
   in
   let rec register n = function
     | "xsd:element" ->
-	let name = check_redef n elts "element" in
+	let name = check_redef n elts_elems "element" in
 	QTable.add elts_elems name n;
 	todo := (fun () -> ignore (resolve_elt name)):: !todo
     | ("xsd:simpleType" | "xsd:complexType") as s ->
-	let name = check_redef n elts "type" in
+	let name = check_redef n typs "type" in
 	let l = if s="xsd:simpleType" then lazy (Simple (parse_simple_type n))
 	else lazy (parse_complex_type_def n) in
 	QTable.add typs name l
@@ -634,7 +633,7 @@ let schema_of_uri uri =
 (* end of parse_root *)
 
     in
-    if List.exists (Ns.equal targetNamespace) !imported_ns then
+    if List.exists (Ns.Uri.equal targetNamespace) !imported_ns then
       print_endline "(already imported)"
     else (
       imported_ns := targetNamespace :: !imported_ns;
