@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 let debug = false
@@ -14,7 +14,7 @@ open Schema_common
 open Schema_types
 open Value
 
-module QTable = Hashtbl.Make(Ns.QName)
+module QTable = Hashtbl.Make(Ns.Label)
 
 let ppf = Format.std_formatter
 
@@ -65,7 +65,7 @@ let concat ctx v = ctx.ctx_current <- Value.concat ctx.ctx_current v
 let append ctx v = ctx.ctx_current <- Value.append ctx.ctx_current v
 
 let xml qname attrs content =
-  Value.Xml (Value.Atom (Atoms.V.of_qname qname), attrs, content)
+  Value.Xml (Value.Atom qname, attrs, content)
 
 
 let peek ctx = 
@@ -133,9 +133,9 @@ let check_nil ctx =
 
 let expect_start_tag ctx tag =
   match next ctx with
-    | E_start_tag t when Ns.QName.equal t tag -> ()
+    | E_start_tag t when Atoms.V.equal t tag -> ()
     | ev -> error (sprintf "Expected tag %s, found %s" 
-		     (Ns.QName.to_string tag) (string_of_event ev))
+		     (Atoms.V.to_string tag) (string_of_event ev))
 
 
 let expect_any_start_tag ctx =
@@ -146,7 +146,7 @@ let expect_any_start_tag ctx =
 
 type attrs = {
   xsi_nil: bool;
-  attrs: (Ns.qname * Utf8.t) list
+  attrs: (Ns.Label.t * Utf8.t) list
 }
 
 let get_bool v = 
@@ -158,7 +158,7 @@ let get_bool v =
 let get_attributes ctx =
   let rec aux attrs nil =
     match peek ctx with
-      | E_attribute (qname,value) when Ns.QName.equal qname xsi_nil_qname ->
+      | E_attribute (qname,value) when Ns.Label.equal qname xsi_nil_label ->
 	  junk ctx;
 	  aux attrs (get_bool value)
       | E_attribute (qname, value) ->
@@ -321,9 +321,9 @@ let rec validate_any_type ctx =
 
 let validate_wildcard ctx w =
   let qname = expect_any_start_tag ctx in
-  if not (Atoms.contains (Atoms.V.of_qname qname) w.wild_first)
+  if not (Atoms.contains qname w.wild_first)
   then error (sprintf "Tag %s is not accepted by the wildcard" 
-		(Ns.QName.to_string qname));
+		(Atoms.V.to_string qname));
   let (attrs, content) = validate_any_type ctx in
   expect_end_tag ctx;
   xml qname attrs content
@@ -360,17 +360,17 @@ let validate_attribute_uses attrs (attr_uses,anyattr) =
 	 with Not_found ->
 	   if anyattr then Value.string_utf8 value
 	   else error 
-	     (sprintf "Unexpected attribute: %s" (Ns.QName.to_string qname))
+	     (sprintf "Unexpected attribute: %s" (Ns.Label.string_of_attr qname))
        in
        attribs := (qname, value) :: !attribs
     ) attrs.attrs;
   if attrs.xsi_nil then
-    attribs := (xsi_nil_qname, Value.vtrue) :: !attribs;
+    attribs := (xsi_nil_label, Value.vtrue) :: !attribs;
   QTable.iter
     (fun qname at ->
        if at.attr_required then  (* check for missing required attributes *)
          error (sprintf "Required attribute %s is missing"
-		  (Ns.QName.to_string qname))
+		  (Ns.Label.string_of_attr qname))
        else  (* add default values *)
          match at.attr_use_cstr with
            | Some (`Default (_,v)) -> attribs := (qname, v) :: !attribs
@@ -401,7 +401,7 @@ and validate_type ctx attrs = function
       if (List.length attrs.attrs > 0) then
 	error "attribute on element with simple content";
       if attrs.xsi_nil then (check_nil ctx; 
-			     Value.vrecord [xsi_nil_qname,Value.vtrue],
+			     Value.vrecord [xsi_nil_label,Value.vtrue],
 			     Value.nil)
       else (empty_record, validate_simple_type st_def (get_string ctx))
   | Complex ct_def -> validate_complex_type ctx attrs ct_def
@@ -420,7 +420,6 @@ and validate_content_type ctx content_type =
   | CT_simple st_def -> 
       validate_simple_type st_def (get_string ctx)
   | CT_model (particle, mixed) ->
-      let mixold = ctx.ctx_mixed in
       let ctx = subctx mixed ctx in
       validate_particle ctx particle;
       get ctx
@@ -430,22 +429,21 @@ and validate_particle ctx particle =
     do_pcdata ctx;
     match peek ctx with
     | E_start_tag qname 
-	when Atoms.contains (Atoms.V.of_qname qname) particle.part_first ->
+	when Atoms.contains qname particle.part_first ->
 	validate_term ctx particle.part_term;
         cont_ok ()
     | ev -> 
 	cont_failure ev
   in
   let rec required = function
-    | 0 -> ()
-    | n ->
+    | 0 -> ()    | n ->
         validate_once
           ~cont_ok:(fun () -> required (pred n))
           ~cont_failure:(fun event ->
 			   if particle.part_nullable then ()
 			   else
-			     error ~ctx (sprintf "Unexpected content: %s"
-					   (string_of_event event)))
+			     error (sprintf "Unexpected content: %s"
+				      (string_of_event event)))
   in
   let rec optional = function
     | None ->
@@ -478,25 +476,20 @@ and validate_choice ctx particles =
   do_pcdata ctx;
   try
     (match peek ctx with
-       | E_start_tag qname -> 
-	   let particle = Atoms.get_map (Atoms.V.of_qname qname) tbl in
-	   validate_particle ctx particle
+       | E_start_tag q -> validate_particle ctx (Atoms.get_map q tbl)
        | _ -> raise Not_found)
   with Not_found ->
     error (sprintf "Cannot choose branch of choice group")
 
 and validate_all_group ctx particles =
-  let tbl = QTable.create 20 in
   let slots = List.map (fun p -> (p, ref None)) particles in
   let tbl = Atoms.mk_map
     (List.map (fun (p,slot) -> p.part_first, (p,slot)) slots) in
   
-  let contents = ref Value.nil in
   let rec aux () =
     match peek ctx with
-      | E_start_tag qname -> 
-	  let qname = next_tag ctx in
-	  let p,slot = Atoms.get_map (Atoms.V.of_qname qname) tbl in
+      | E_start_tag q ->
+	  let p,slot = Atoms.get_map q tbl in
 	  (match !slot with
 	    | Some x -> ()
 	    | None -> 
@@ -548,7 +541,7 @@ let validate_type def value =
       let attrs = get_attributes ctx in
       let (attrs, content) = validate_complex_type ctx attrs ct_def in
       expect_end_tag ctx;
-      Value.Xml (Value.Atom (Atoms.V.of_qname start_tag), attrs, content)
+      Value.Xml (Value.Atom start_tag, attrs, content)
 
 (*
 let validate_attribute decl value =
@@ -615,3 +608,16 @@ let validate_model_group { mg_def = mg } value =
   get ctx
 
 
+
+type t =
+  | VAttrGp of attribute_group_definition
+  | VModelGp of model_group_definition
+  | VType of type_definition
+  | VElem of element_declaration
+
+let run s v =
+  match s with
+    | VAttrGp x -> validate_attribute_group x v
+    | VModelGp x -> validate_model_group x v
+    | VType x -> validate_type x v
+    | VElem x -> validate_element x v
