@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Encodings
@@ -16,9 +16,9 @@ val _may_attr: string -> node -> Utf8.t option
 val _is_attr: string -> node -> string -> bool
 val _attr: string -> node -> Utf8.t
 
-val _resolve_qname: node -> Utf8.t -> Ns.qname
-val _may_qname_attr: string -> node -> Ns.qname option
-val _qname_attr: string -> node -> Ns.qname
+val _resolve_qname: node -> Utf8.t -> Ns.QName.t
+val _may_qname_attr: string -> node -> Ns.QName.t option
+val _qname_attr: string -> node -> Ns.QName.t
 
 val _tag: node -> string
 val _elems: string -> node -> node list
@@ -28,5 +28,5 @@ val _may_elem: string -> node -> node option
 val _iter_elems: node -> (node -> string -> unit) -> unit
 
 
-val xsd: Ns.t
-val xsi: Ns.t
+val xsd: Ns.Uri.t
+val xsi: Ns.Uri.t
