@@ -1,17 +1,17 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 module U = Encodings.Utf8
 
-module Id = Pool.Weak(Ns.QName)
+module Id = Ns.QName
 type id = Id.t
-let ident = Id.mk
-let value = Id.value
-let to_string id = Ns.QName.to_string (Id.value id)
-let print ppf id = Ns.QName.print ppf (Id.value id)
+let ident x = x
+let value x = x
+let to_string = Id.to_string
+let print = Id.print
 
 module IdSet = SortedList.Make(Id)
 module IdMap = IdSet.Map
@@ -21,11 +21,10 @@ type fv = IdSet.t
 
 
 
-module Label = Ns.QName
-
-module LabelPool = Pool.Make(Label)
-module LabelSet = SortedList.Make(LabelPool)
+(* TODO: put following decl somewhere else *)
+module Label = Ns.Label
+module LabelSet = SortedList.Make(Ns.Label)
 module LabelMap = LabelSet.Map
 
-type label = LabelPool.t
+type label = Ns.Label.t
 type 'a label_map = 'a LabelMap.map
