@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Big_int
@@ -23,10 +23,7 @@ let check i = ()
 
 let from_int i = big_int_of_int i
 let from_bigint i = i
-
-(* TODO: better serialization of bigints !!! *)
-let serialize t i = Serialize.Put.string t (string_of_big_int i)
-let deserialize t = big_int_of_string (Serialize.Get.string t)
+  
 
 let mk = big_int_of_string
 let to_string = string_of_big_int
@@ -40,6 +37,7 @@ let div = div_big_int
 let modulo  = mod_big_int
 let succ = succ_big_int
 let pred = pred_big_int
+let negat t = sub_big_int zero_big_int t
 
 let lt = lt_big_int
 let gt = gt_big_int
@@ -48,6 +46,11 @@ let zero = big_int_of_int 0
 let one = big_int_of_int 1
 let minus_one = big_int_of_int (-1)
 let is_zero = equal zero
+
+let from_int32 i = mk (Int32.to_string i)
+let from_int64 i = mk (Int64.to_string i)
+let to_int32 i = Int32.of_string (to_string i)
+let to_int64 i = Int64.of_string (to_string i)
 end
 
 type interval = 
@@ -68,37 +71,6 @@ let rec check = function
       check rem
   | _ -> assert false
 
-let serialize_interval t = function
-  | Bounded (i1,i2) ->
-      Serialize.Put.bits 2 t 0;
-      V.serialize t i1;
-      V.serialize t i2
-  | Left i ->
-      Serialize.Put.bits 2 t 1;
-      V.serialize t i;
-  | Right i ->
-      Serialize.Put.bits 2 t 2;
-      V.serialize t i
-  | Any ->
-      Serialize.Put.bits 2 t 3
-
-let deserialize_interval t =
-  match Serialize.Get.bits 2 t with
-    | 0 ->
-	let i1 = V.deserialize t in
-	let i2 = V.deserialize t in
-	Bounded (i1,i2)
-    | 1 ->
-	Left (V.deserialize t)
-    | 2 ->
-	Right (V.deserialize t)
-    | 3 ->
-	Any
-    | _ -> assert false
-
-let serialize = Serialize.Put.list serialize_interval
-let deserialize = Serialize.Get.list deserialize_interval
-
 let rec compare l1 l2 =
   match (l1,l2) with
     | [], [] -> 0
@@ -389,6 +361,9 @@ let mul l1 l2 =
     ) empty l1
 
 
+let div i1 i2 = any
+let modulo i1 i2 = any
+
 let dmp s i =
   let ppf = Format.std_formatter in
   Format.fprintf ppf "%s = [ " s;
@@ -416,3 +391,6 @@ let cap i1 i2 =
   cap i1 i2
 
 *)
+
+let int32 = bounded (V.from_int32 Int32.min_int) (V.from_int32 Int32.max_int)
+let int64 = bounded (V.from_int64 Int64.min_int) (V.from_int64 Int64.max_int)
