@@ -1,16 +1,17 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
+(*  Copyright CNRS,INRIA, 2003,2004,2005 (see LICENSE for details)        *)
 (**************************************************************************)
 
 open Ident
 open Encodings
 
-(* TODO:
-   - I store hash in types to avoid computing it several times.
-     Does not seem to help a lot.
-*)
+let count = ref 0
+		
+let () =
+  Stats.register Stats.Summary
+    (fun ppf -> Format.fprintf ppf "Allocated type nodes:%i@\n" !count)
 
 (*
 To be sure not to use generic comparison ...
@@ -21,54 +22,9 @@ let (<=) : int -> int -> bool = (<=)
 let (<>) : int -> int -> bool = (<>)
 let compare = 1
 
-
-module CompUnit = struct   
-  include Pool.Make(Utf8)
-  module Tbl = Inttbl
-
-  let pervasives = mk (U.mk "Pervasives") 
-  let current = ref dummy_min
-  let get_current () =
-    assert (!current != dummy_min);
-    !current
-
-  let print_qual ppf t =
-    if (t != !current) && (t != pervasives) then
-      Format.fprintf ppf "%a." U.print (value t)
-      
-  let close_serialize_ref = ref (fun () -> assert false)
-      
-  let depend = Inttbl.create ()
-      
-  let serialize t cu =
-    if (cu != pervasives) && (cu != !current) then Inttbl.add depend cu ();
-    serialize t cu
-
-  let close_serialize () =
-    !close_serialize_ref ();
-    let deps = Inttbl.fold depend (fun cu () l -> cu :: l) [] in
-    Inttbl.clear depend;
-    deps
-
-
-  let stack = ref []
-		
-  let enter i = 
-    stack := !current :: !stack; current := i
-		  
-  let leave () =
-    match !stack with
-      | hd::tl -> current := hd; stack := tl
-      | _ -> assert false
-
-
-  let () = enter pervasives
-end	  
-
-
 type const = 
   | Integer of Intervals.V.t
-  | Atom of Atoms.V.t 
+  | Atom of Atoms.V.t
   | Char of Chars.V.t
   | Pair of const * const
   | Xml of const * const
@@ -78,67 +34,8 @@ type const =
 module Const = struct
   type t = const
 
-  let rec check = function
-    | Integer i -> Intervals.V.check i
-    | Atom i -> Atoms.V.check i
-    | Char i -> Chars.V.check i
-    | Pair (x,y) | Xml (x,y) -> check x; check y
-    | Record l -> LabelMap.iter check l
-    | String (i,j,s,q) -> U.check s; check q
-
-  let dump ppf _ =
-    Format.fprintf ppf "<Types.Const.t>"
-
-  let rec serialize s = function
-    | Integer x ->
-	Serialize.Put.bits 3 s 0;
-	Intervals.V.serialize s x
-    | Atom x ->
-	Serialize.Put.bits 3 s 1;
-	Atoms.V.serialize s x
-    | Char x ->
-	Serialize.Put.bits 3 s 2;
-	Chars.V.serialize s x
-    | Pair (x,y) ->
-	Serialize.Put.bits 3 s 3;
-	serialize s x;
-	serialize s y
-    | Xml (x,y) ->
-	Serialize.Put.bits 3 s 4;
-	serialize s x;
-	serialize s y
-    | Record r ->
-	Serialize.Put.bits 3 s 5;
-	LabelMap.serialize serialize s r
-    | String (i,j,st,q) ->
-	Serialize.Put.bits 3 s 6;
-	U.serialize_sub s st i j;
-	serialize s q
-
-  let rec deserialize s =
-    match Serialize.Get.bits 3 s with
-      | 0 ->
-	  Integer (Intervals.V.deserialize s)
-      | 1 ->
-	  Atom (Atoms.V.deserialize s)
-      | 2 ->
-	  Char (Chars.V.deserialize s)
-      | 3 ->
-	  let x = deserialize s in
-	  let y = deserialize s in
-	  Pair (x,y)
-      | 4 ->
-	  let x = deserialize s in
-	  let y = deserialize s in
-	  Xml (x,y)
-      | 5 ->
-	  Record (LabelMap.deserialize deserialize s)
-      | 6 ->
-	  let st = U.deserialize s in
-	  let q = deserialize s in
-	  String (U.start_index st, U.end_index st, st, q)
-      | _ ->
-	  assert false
+  let check _ = ()
+  let dump ppf _ = Format.fprintf ppf "<Types.Const.t>"
 
   let rec compare c1 c2 = match (c1,c2) with
     | Integer x, Integer y -> Intervals.V.compare x y
@@ -219,7 +116,6 @@ sig
   but a  bug (?) in OCaml 3.07 makes it impossible
 *)
   type t = {
-    mutable hash: int;
     atoms : Atoms.t;
     ints  : Intervals.t;
     chars : Chars.t;
@@ -236,12 +132,9 @@ sig
   val equal: t -> t -> bool
   val hash: t -> int
   val compare:t -> t -> int
-  val serialize: t Serialize.Put.f
-  val deserialize: t Serialize.Get.f
 end =
 struct
   type t = {
-    mutable hash: int;
     atoms : Atoms.t;
     ints  : Intervals.t;
     chars : Chars.t;
@@ -253,11 +146,17 @@ struct
     absent: bool
   }
 
-  let dump ppf _ =
-    Format.fprintf ppf "<Types.Descr.t>"
+  let print_lst ppf =
+    List.iter (fun f -> f ppf; Format.fprintf ppf " |")
+
+  let dump ppf d =
+    Format.fprintf ppf "<types atoms(%a) times(%a) record(%a) xml(%a)>"
+      print_lst (Atoms.print d.atoms)
+      BoolPair.dump d.times
+      BoolRec.dump d.record
+      BoolPair.dump d.xml
 
   let empty = { 
-    hash = 0;
     times = BoolPair.empty; 
     xml   = BoolPair.empty; 
     arrow = BoolPair.empty; 
@@ -297,19 +196,16 @@ struct
     else 0
       
   let hash a =
-    if a.hash <> 0 then a.hash else (
-      let accu = Chars.hash a.chars in
-      let accu = 17 * accu + Intervals.hash a.ints in
-      let accu = 17 * accu + Atoms.hash a.atoms in
-      let accu = 17 * accu + BoolPair.hash a.times in
-      let accu = 17 * accu + BoolPair.hash a.xml in
-      let accu = 17 * accu + BoolPair.hash a.arrow in
-      let accu = 17 * accu + BoolRec.hash a.record in
-      let accu = 17 * accu + Abstract.hash a.abstract in
-      let accu = if a.absent then accu+5 else accu in
-      a.hash <- accu;
-      accu
-    )
+    let accu = Chars.hash a.chars in
+    let accu = 17 * accu + Intervals.hash a.ints in
+    let accu = 17 * accu + Atoms.hash a.atoms in
+    let accu = 17 * accu + BoolPair.hash a.times in
+    let accu = 17 * accu + BoolPair.hash a.xml in
+    let accu = 17 * accu + BoolPair.hash a.arrow in
+    let accu = 17 * accu + BoolRec.hash a.record in
+    let accu = 17 * accu + Abstract.hash a.abstract in
+    let accu = if a.absent then accu+5 else accu in
+    accu
 
   let check a =
     Chars.check a.chars;
@@ -323,118 +219,27 @@ struct
     ()
 
 
-  let serialize t a =
-    Chars.serialize t a.chars;
-    Intervals.serialize t a.ints;
-    Atoms.serialize t a.atoms;
-    BoolPair.serialize t a.times;
-    BoolPair.serialize t a.xml;
-    BoolPair.serialize t a.arrow;
-    BoolRec.serialize t a.record;
-    Abstract.serialize t a.abstract;
-    Serialize.Put.bool t a.absent 
-
-  let deserialize t =
-    let chars = Chars.deserialize t in
-    let ints = Intervals.deserialize t in
-    let atoms = Atoms.deserialize t in
-    let times = BoolPair.deserialize t in
-    let xml = BoolPair.deserialize t in
-    let arrow = BoolPair.deserialize t in
-    let record = BoolRec.deserialize t in
-    let abstract = Abstract.deserialize t in
-    let absent = Serialize.Get.bool t in
-    let d = { hash=0; 
-      chars = chars; ints = ints; atoms = atoms; times = times; xml = xml;
-      arrow = arrow; record = record; abstract = abstract; absent = absent } in
-    check d;
-    d
-   
-    
 end
 and Node :
 sig
-  type t = { id : int; comp_unit: CompUnit.t; mutable descr : Descr.t }
+  type t = { id : int; cu: Compunit.t; mutable descr : Descr.t }
   val dump: Format.formatter -> t -> unit
   val check: t -> unit
   val equal: t -> t -> bool
   val hash: t -> int
   val compare:t -> t -> int
-  val serialize: t Serialize.Put.f
-  val deserialize: t Serialize.Get.f
   val mk: int -> Descr.t -> t
 end =
 
 struct
-  type t = { id : int; comp_unit : CompUnit.t; mutable descr : Descr.t }
+  type t = { id : int; cu: Compunit.t; mutable descr : Descr.t }
   let check n = ()
-  let dump ppf n = failwith "Types.Node.dump"
-  let hash x = x.id + 17 * x.comp_unit
+  let dump ppf n = Format.fprintf ppf "X%i" n.id
+  let hash x = x.id + Compunit.hash x.cu
   let compare x y = 
-    let c = x.id - y.id in
-    if c = 0 then x.comp_unit - y.comp_unit else c
-  let equal x y = x == y
-
-  let serialize_memo = Inttbl.create ()
-  let counter_serialize = ref 0
-
-  let () =
-    CompUnit.close_serialize_ref := 
-    (fun () ->
-       Inttbl.clear serialize_memo;
-       counter_serialize := 0)
-
-  let serialize t n = 
-    if n.comp_unit == !CompUnit.current then (
-      Serialize.Put.bool t true;
-      try 
-	let i = Inttbl.find serialize_memo n.id in
-	Serialize.Put.int t i
-      with Not_found ->
-	let i = !counter_serialize in
-	incr counter_serialize;
-	Inttbl.add serialize_memo n.id i;
-	Serialize.Put.int t i;
-	Descr.serialize t n.descr
-    ) else (
-      Serialize.Put.bool t false;
-      CompUnit.serialize t n.comp_unit;
-      Serialize.Put.int t n.id
-    )
-      
-
-  let deserialize_memo = Inttbl.create ()
-
-  let find_tbl id =
-    try Inttbl.find deserialize_memo id
-    with Not_found ->
-      let tbl = Inttbl.create () in
-      Inttbl.add deserialize_memo id tbl;
-      tbl
-
-  let mk id d =
-    let n = { id = id; comp_unit = !CompUnit.current; descr = d } in
-    if !CompUnit.current == CompUnit.pervasives then
-      Inttbl.add (find_tbl CompUnit.pervasives) n.id n;
-    n
-
-  let deserialize t = 
-    if Serialize.Get.bool t then
-      let i = Serialize.Get.int t in
-      let tbl = find_tbl !CompUnit.current in
-      try Inttbl.find tbl i
-      with Not_found ->
-	let n = { id = i; comp_unit = !CompUnit.current;
-		  descr = Descr.empty } in
-	Inttbl.add tbl i n;
-	n.descr <- Descr.deserialize t;
-	n
-    else
-      let cu = CompUnit.deserialize t in
-      let i = Serialize.Get.int t in
-      try Inttbl.find (Inttbl.find deserialize_memo cu) i 
-      with Not_found -> assert false
-
+    let c = x.id - y.id in if c = 0 then Compunit.compare x.cu y.cu else c
+  let equal x y = x==y || (x.id == y.id && (Compunit.equal x.cu y.cu))
+  let mk id d = { id = id; cu = Compunit.current (); descr = d }
 end
 
 (* See PR#2920 in OCaml BTS *)
@@ -446,18 +251,16 @@ struct
   let equal x = Node.equal x
   let hash x = Node.hash x
   let compare x = Node.compare x
-  let serialize x = Node.serialize x
-  let deserialize x = Node.deserialize x
 end
 
 
 (* It is also possible to use Boolean instead of Bool here;
    need to analyze when each one is more efficient *)
 and BoolPair : Bool.S with type elem = Node.t * Node.t = 
-Bool.Make(Custom.Pair(NodeT)(NodeT))
+(*Bool.Simplify*)(Bool.Make)(Custom.Pair(NodeT)(NodeT))
 
 and BoolRec : Bool.S with type elem = bool * Node.t label_map =
-Bool.Make(Custom.Pair(Custom.Bool)(LabelSet.MakeMap(NodeT)))
+(*Bool.Simplify*)(Bool.Make)(Custom.Pair(Custom.Bool)(LabelSet.MakeMap(NodeT)))
 
 module DescrHash = Hashtbl.Make(Descr)
 module DescrMap = Map.Make(Descr)
@@ -470,30 +273,34 @@ include Descr
 
 let forward_print = ref (fun _ _ -> assert false)
 
-let hash_cons = DescrHash.create 17000  
-
-let count = State.ref "Types.count" 0
-		
-let () =
-  Stats.register Stats.Summary
-    (fun ppf -> Format.fprintf ppf "Allocated type nodes:%i@\n" !count)
-      
 let make () = 
   incr count; 
   Node.mk !count empty
 
+(*
+let hash_cons = DescrHash.create 17000  
+
 let define n d = 
   DescrHash.add hash_cons d n; 
   n.Node.descr <- d
+
 let cons d = 
   try DescrHash.find hash_cons d 
   with Not_found ->
     incr count; 
     let n = Node.mk !count d in
     DescrHash.add hash_cons d n; n  
+*)
+
+let define n d = 
+  n.Node.descr <- d
+
+let cons d = 
+  incr count; 
+  Node.mk !count d
+
 
 let any =  {
-  hash = 0;
   times = BoolPair.full; 
   xml   = BoolPair.full; 
   arrow = BoolPair.full; 
@@ -508,30 +315,28 @@ let any =  {
 
 let non_constructed =
   { any with  
-      hash = 0;
       times = empty.times; xml = empty.xml; record = empty.record }
      
 let non_constructed_or_absent = 
-  { non_constructed with hash = 0; absent = true }
+  { non_constructed with absent = true }
 	     
-let interval i = { empty with hash = 0; ints = i }
-let times x y = { empty with hash = 0; times = BoolPair.atom (x,y) }
-let xml x y = { empty with hash = 0; xml = BoolPair.atom (x,y) }
-let arrow x y = { empty with hash = 0; arrow = BoolPair.atom (x,y) }
+let interval i = { empty with ints = i }
+let times x y = { empty with times = BoolPair.atom (x,y) }
+let xml x y = { empty with xml = BoolPair.atom (x,y) }
+let arrow x y = { empty with arrow = BoolPair.atom (x,y) }
 let record label t = 
-  { empty with hash = 0; 
+  { empty with 
       record = BoolRec.atom (true,LabelMap.singleton label t) }
-let record' (x : bool * node Ident.label_map) =
-  { empty with hash = 0; record = BoolRec.atom x }
-let atom a = { empty with hash = 0; atoms = a }
-let char c = { empty with hash = 0; chars = c }
-let abstract a = { empty with hash = 0; abstract = a }
+let record_fields (x : bool * node Ident.label_map) =
+  { empty with record = BoolRec.atom x }
+let atom a = { empty with atoms = a }
+let char c = { empty with chars = c }
+let abstract a = { empty with abstract = a }
 
 let get_abstract t = t.abstract
       
 let cup x y = 
   if x == y then x else {
-    hash = 0;
     times = BoolPair.cup x.times y.times;
     xml   = BoolPair.cup x.xml y.xml;
     arrow = BoolPair.cup x.arrow y.arrow;
@@ -545,7 +350,6 @@ let cup x y =
     
 let cap x y = 
   if x == y then x else {
-    hash = 0;
     times = BoolPair.cap x.times y.times;
     xml   = BoolPair.cap x.xml y.xml;
     record= BoolRec.cap x.record y.record;
@@ -559,7 +363,6 @@ let cap x y =
     
 let diff x y = 
   if x == y then empty else {
-    hash = 0;
     times = BoolPair.diff x.times y.times;
     xml   = BoolPair.diff x.xml y.xml;
     arrow = BoolPair.diff x.arrow y.arrow;
@@ -598,8 +401,8 @@ let rec constant = function
   | Atom a -> atom (Atoms.atom a)
   | Char c -> char (Chars.atom c)
   | Pair (x,y) -> times (const_node x) (const_node y)
-  | Xml (x,y) -> times (const_node x) (const_node y)
-  | Record x -> record' (false ,LabelMap.map const_node x)
+  | Xml (x,y) -> xml (const_node x) (const_node y)
+  | Record x -> record_fields (false ,LabelMap.map const_node x)
   | String (i,j,s,c) ->
       if U.equal_index i j then constant c
       else 
@@ -612,10 +415,10 @@ let neg x = diff any x
 let any_node = cons any
 let empty_node = cons empty
 
-module LabelS = Set.Make(LabelPool)
+module LabelS = Set.Make(Label)
 
-let any_or_absent = { any with hash=0; absent = true } 
-let only_absent = { empty with hash=0; absent = true }
+let any_or_absent = { any with absent = true } 
+let only_absent = { empty with absent = true }
 
 let get_record r =
   let labs accu (_,r) = 
@@ -675,7 +478,7 @@ let cap_product any_left any_right l =
     (fun (d1,d2) (t1,t2) -> (cap_t d1 t1, cap_t d2 t2))
     (any_left,any_right)
     l
-let any_pair = { empty with hash = 0; times = any.times }
+let any_pair = { empty with times = any.times }
 
 
 let rec exists max f =
@@ -736,7 +539,7 @@ let guard a f n =
 module ClearlyEmpty = 
 struct
 
-let memo = DescrHash.create 33000
+let memo = DescrHash.create 8191
 let marks = ref [] 
 
 let rec slot d =
@@ -831,12 +634,15 @@ let clearly_disjoint t1 t2 =
 
 (* TODO: need to invesigate when ClearEmpty is a good thing... *)
 
-let memo = DescrHash.create 33000
+let memo = DescrHash.create 8191
 let marks = ref [] 
 
 let count_subtype = Stats.Counter.create "Subtyping internal loop" 
 
+let complex = ref 0
+
 let rec slot d =
+  incr complex;
   Stats.Counter.incr count_subtype; 
   if not ((Intervals.is_empty d.ints) && 
 	  (Atoms.is_empty d.atoms) &&
@@ -859,8 +665,8 @@ let rec slot d =
 
 and check_times (left,right) s =
   let rec aux accu1 accu2 right s = match right with
-    | (t1,t2)::right ->
-	let t1 = descr t1 and t2 = descr t2 in
+    | (n1,n2)::right ->
+	let t1 = descr n1 and t2 = descr n2 in
 	if trivially_disjoint accu1 t1 || 
 	   trivially_disjoint accu2 t2 then (
 	     aux accu1 accu2 right s )
@@ -878,8 +684,8 @@ and check_times (left,right) s =
 
 and check_xml (left,right) s =
   let rec aux accu1 accu2 right s = match right with
-    | (t1,t2)::right ->
-	let t1 = descr t1 and t2 = descr t2 in
+    | (n1,n2)::right ->
+	let t1 = descr n1 and t2 = descr n2 in
 	if clearly_disjoint accu1 t1 || 
 	   trivially_disjoint accu2 t2 then (
 	     aux accu1 accu2 right s )
@@ -962,6 +768,20 @@ let is_empty d =
   if ClearlyEmpty.is_empty d then (Printf.eprintf "!\n"; true) else is_empty d
 *)  
 
+(*
+let is_empty d =
+(*  Format.fprintf Format.std_formatter "complex=%i@."
+	  !complex; *)
+  if !complex = 0 then
+    (let r = is_empty d in
+     if !complex > 100 then
+       (let c = !complex in
+	Format.fprintf Format.std_formatter "is_empty (%i)@." c
+	  (*Descr.dump (*!forward_print*) d*));
+     complex := 0; r)
+  else is_empty d
+*)
+
 let non_empty d = 
   not (is_empty d)
 
@@ -979,8 +799,8 @@ struct
 
   let other ?(kind=`Normal) d = 
     match kind with
-      | `Normal -> { d with hash = 0; times = empty.times }
-      | `XML -> { d with hash = 0; xml = empty.xml }
+      | `Normal -> { d with times = empty.times }
+      | `XML -> { d with xml = empty.xml }
 
   let is_product ?kind d = is_empty (other ?kind d)
 
@@ -1100,6 +920,7 @@ struct
       | `XML -> normal_xml d.xml
 
 
+(*
   let merge_same_2 r =
     let r = 
       List.fold_left 
@@ -1108,7 +929,7 @@ struct
 	   DescrMap.add t2 (cup t t1) accu
 	) DescrMap.empty r in
     DescrMap.fold (fun t2 t1 accu -> (t1,t2)::accu) r []
-	 
+*)	 
 
   let constraint_on_2 n t1 =
     List.fold_left 
@@ -1126,16 +947,16 @@ struct
 	| (s1,s2) :: rem -> (s1,s2) :: (aux rem (t1,t2)) in
     List.fold_left aux [] l
 
-  let any = { empty with hash = 0; times = any.times }
-  and any_xml = { empty with hash = 0; xml = any.xml }
+  let any = { empty with times = any.times }
+  and any_xml = { empty with xml = any.xml }
   let is_empty d = d == []
   let any_of = function `XML -> any_xml | `Normal -> any
 end
 
 module Record = 
 struct
-  let has_record d = not (is_empty { empty with hash= 0; record = d.record })
-  let or_absent d = { d with hash = 0; absent = true }
+  let has_record d = not (is_empty { empty with record = d.record })
+  let or_absent d = { d with absent = true }
   let absent = or_absent empty
   let any_or_absent = or_absent any
   let any_or_absent_node = cons any_or_absent
@@ -1154,7 +975,7 @@ struct
   end
   module R = struct
     type t = descr
-    let any = { empty with hash = 0; record = any.record }
+    let any = { empty with record = any.record }
     let cap = cap
     let cup = cup
     let diff = diff
@@ -1163,11 +984,11 @@ struct
   end
   module TR = Normal.Make(T)(R)
 
-  let any_record = { empty with hash = 0; record = BoolRec.full }
+  let any_record = { empty with record = BoolRec.full }
 
   let atom o l = 
     if o && LabelMap.is_empty l then any_record else
-    { empty with hash = 0; record = BoolRec.atom (o,l) }
+    { empty with record = BoolRec.atom (o,l) }
 
   type zor = Pair of descr * descr | Any
 
@@ -1179,10 +1000,10 @@ struct
       with Not_found -> 
 	if o then
 	  if LabelMap.is_empty r then Any else
-	    Pair (any_or_absent, { empty with hash=0; record = BoolRec.atom (o,r) })
+	    Pair (any_or_absent, { empty with record = BoolRec.atom (o,r) })
 	else
 	  Pair (absent,
-		{ empty with hash = 0; record = BoolRec.atom (o,r) })
+		{ empty with record = BoolRec.atom (o,r) })
     in
     List.fold_left 
       (fun b (p,n) ->
@@ -1218,7 +1039,7 @@ struct
 
   let project_opt d l =
     let t = pi l d in
-    { t with hash = 0; absent = false }
+    { t with absent = false }
 
   let condition d l t =
     TR.pi2_restricted t (split d l)
@@ -1238,10 +1059,9 @@ struct
     !res
 
   let first_label d =
-    let min = ref LabelPool.dummy_max in
-    let aux (_,r) = 
-      match LabelMap.get r with
-	  (l,_)::_ -> if (l:int) < !min then min := l | _ -> () in
+    let min = ref Label.dummy in
+    let aux (_,r) = match LabelMap.get r with
+	(l,_)::_ -> min := Label.min l !min | _ -> () in
     BoolRec.iter aux d.record;
     !min
 
@@ -1276,20 +1096,20 @@ struct
   let merge d1 d2 = 
     let res = ref empty in
     let rec aux accu d1 d2 =
-      let l = min (first_label d1) (first_label d2) in
-      if l = LabelPool.dummy_max then
+      let l = Label.min (first_label d1) (first_label d2) in
+      if l == Label.dummy then
 	let (some1,none1) = empty_cases d1 
 	and (some2,none2) = empty_cases d2 in
-	let none = none1 && none2 and some = some1 || some2 in
+	let _none = none1 && none2 and some = some1 || some2 in
 	let accu = LabelMap.from_list (fun _ _ -> assert false) accu in
 	(* approx for the case (some && not none) ... *)
-	res := cup !res (record' (some, accu))
+	res := cup !res (record_fields (some, accu))
       else
 	let l1 = split d1 l and l2 = split d2 l in
 	let loop (t1,d1) (t2,d2) =
 	  let t = 
 	    if t2.absent 
-	    then cup t1 { t2 with hash = 0; absent = false } 
+	    then cup t1 { t2 with absent = false } 
 	    else t2 
 	  in
 	  aux ((l,cons t)::accu) d1 d2
@@ -1300,18 +1120,18 @@ struct
     aux [] d1 d2;
     !res
 
-  let any = { empty with hash = 0; record = any.record }
+  let any = { empty with record = any.record }
 
   let get d =
     let rec aux r accu d =
       let l = first_label d in
-      if l == LabelPool.dummy_max then
+      if l == Label.dummy then
 	let (o1,o2) = empty_cases d in 
 	if o1 || o2 then (LabelMap.from_list_disj r,o1,o2)::accu else accu
       else
 	List.fold_left 
 	  (fun accu (t1,t2) -> 
-	     let x = (t1.absent, { t1 with hash = 0; absent = false }) in
+	     let x = (t1.absent, { t1 with absent = false }) in
 	     aux ((l,x)::r) accu t2)
 	  accu
 	  (split d l)
@@ -1330,12 +1150,10 @@ struct
     | Xml (x,y) -> Format.fprintf ppf "XML(%a,%a)" print_const x print_const y
     | Record r -> 
 	Format.fprintf ppf "Record{";
-	List.iter 
-	  (fun (l,c) -> 
-	     Format.fprintf ppf "%a : %a; " 
-	     Label.print (LabelPool.value l)
-	     print_const c)
-	  (LabelMap.get r);
+	LabelMap.iteri
+	  (fun l c -> 
+	     Format.fprintf ppf "%a : %a; " Label.print_attr l print_const c)
+	  r;
 	Format.fprintf ppf "}"
     | String (i,j,s,c) ->
 	Format.fprintf ppf "\"%a\" %a"
@@ -1352,7 +1170,7 @@ struct
 
   let is_regexp t = subtype t seqs_descr
 
-  type gname = CompUnit.t * Ns.qname
+  type gname = string * Ns.QName.t
 
   type nd = { id : int; 
 	     mutable def : d list; 
@@ -1383,10 +1201,10 @@ struct
 
   module DescrPairMap = Map.Make(Custom.Pair(Descr)(Descr))
 
-  let named = State.ref "Types.Print.named" DescrMap.empty
-  let named_xml = State.ref "Types.Print.named_xml"  DescrPairMap.empty
-  let register_global cu (name : Ns.qname) d = 
-    if equal { d with hash = 0; xml = BoolPair.empty } empty then 
+  let named = ref DescrMap.empty
+  let named_xml = ref DescrPairMap.empty
+  let register_global cu (name : Ns.QName.t) d = 
+    if equal { d with xml = BoolPair.empty } empty then 
       (let l = (*Product.merge_same_2*) (Product.get ~kind:`XML d) in
        match l with
 	 | [(t1,t2)] -> 
@@ -1398,7 +1216,7 @@ struct
     else named := DescrMap.add d (cu,name) !named
 	
   let unregister_global d =
-    if equal { d with hash = 0; xml = BoolPair.empty } empty then
+    if equal { d with xml = BoolPair.empty } empty then
       (let l = Product.get ~kind:`XML d in
       match l with
 	 | [(t1,t2)] -> 
@@ -1419,7 +1237,7 @@ struct
 
   let trivial_rec b = 
     b == BoolRec.empty || 
-    (is_empty { empty with hash = 0; record = BoolRec.diff BoolRec.full b})
+    (is_empty { empty with record = BoolRec.diff BoolRec.full b})
 
   let trivial_pair b = b == BoolPair.empty || b == BoolPair.full
 
@@ -1451,14 +1269,14 @@ struct
 	DescrHash.add memo d s;
 	s
       with Not_found ->
-	if d.absent then alloc [Abs (prepare ({d with hash=0; absent=false}))]
+	if d.absent then alloc [Abs (prepare ({d with absent=false}))]
 	else if worth_complement d 
 	then alloc [Neg (prepare (neg d))]
 	else let slot = alloc [] in
 	if not (worth_abbrev d) then slot.state <- `Expand;
 	DescrHash.add memo d slot;
 	let (seq,not_seq) =
-	  if (subtype { empty with hash = 0; times = d.times } seqs_descr) then
+	  if (subtype { empty with times = d.times } seqs_descr) then
 	    (cap d seqs_descr, diff d seqs_descr)
 	  else 
 	    (empty, d) in
@@ -1477,9 +1295,9 @@ struct
 	     with Not_found ->
 	       let tag = 
 		 match Atoms.print_tag t1.atoms with
-		   | Some a when is_empty { t1 with hash=0; atoms = Atoms.empty } -> `Tag a
+		   | Some a when is_empty { t1 with atoms = Atoms.empty } -> `Tag a
 		   | _ -> `Type (prepare t1) in
-	       assert (equal { t2 with hash=0; times = empty.times } empty);
+	       assert (equal { t2 with times = empty.times } empty);
 	       List.iter
 		 (fun (ta,tb) -> 
 		    add (Xml (tag, prepare ta, prepare tb)))
@@ -1553,7 +1371,7 @@ struct
     | Pretty.Trans t -> assign_name t
 
   let print_gname ppf (cu,n) = 
-    Format.fprintf ppf "%a%a" CompUnit.print_qual cu Ns.QName.print n
+    Format.fprintf ppf "%s%a" cu Ns.QName.print n
 
   let rec do_print_slot pri ppf s =
     match s.state with
@@ -1621,7 +1439,7 @@ struct
       (fun (l,(o,t)) ->
 	 let opt = if o then "?" else "" in
 	 Format.fprintf ppf "@ @[%a=%s@]%a"
-	   Label.print (LabelPool.value l) opt (do_print_slot 0) t
+	   Label.print_attr l opt (do_print_slot 0) t
       ) (LabelMap.get r);
     if not none then Format.fprintf ppf "@ (+others)";
     if some then Format.fprintf ppf " ..";
@@ -1705,6 +1523,16 @@ struct
   let print_node ppf n = print ppf (descr n)
 
   let () = forward_print := print
+
+  let print_to_string f x =
+    let b = Buffer.create 1024 in
+    let ppf = Format.formatter_of_buffer b in
+    f ppf x;
+    Format.pp_print_flush ppf ();
+    Buffer.contents b
+
+  let to_string t = print_to_string print t
+    
 end
 
 module Positive =
@@ -1767,7 +1595,7 @@ let rec rec_normalize d =
     in
     let record = d.record
     in
-    define n { d with hash=0; times = times; xml = xml; record = record };
+    define n { d with times = times; xml = xml; record = record };
     n
 
 let normalize n =
@@ -1794,7 +1622,7 @@ struct
   let sample t =
     let (left,right) = List.find check_line_non_empty (BoolPair.get t.arrow) in
     List.fold_left (fun accu (t,s) -> cap accu (arrow t s)) 
-      { empty with hash=0; arrow = any.arrow } left
+      { empty with arrow = any.arrow } left
 	
       
   let check_strenghten t s =
@@ -1805,7 +1633,7 @@ struct
       | (p,n) :: rem ->
 	  if (List.for_all (fun (a,b) -> check_simple left a b) p) &&
 	    (List.for_all (fun (a,b) -> not (check_simple left a b)) n) then
-	      { empty with hash=0; arrow = Obj.magic [ (SortedList.cup left p, n) ] }  (* rework this ! *)
+	      { empty with arrow = Obj.magic [ (SortedList.cup left p, n) ] }  (* rework this ! *)
 	  else aux rem
     in
     aux (BoolPair.get s.arrow)
@@ -1885,7 +1713,7 @@ struct
       )
       empty arr
 
-  let any = { empty with hash=0; arrow = any.arrow }
+  let any = { empty with arrow = any.arrow }
   let is_empty (_,arr) = arr == []
 end
   
@@ -1893,20 +1721,20 @@ end
 module Int = struct
   let has_int d i = Intervals.contains i d.ints
   let get d = d.ints
-  let any = { empty with hash=0; ints = any.ints }
+  let any = { empty with ints = any.ints }
 end
 
 module Atom = struct
   let has_atom d a = Atoms.contains a d.atoms
   let get d = d.atoms
-  let any = { empty with hash=0; atoms = any.atoms }
+  let any = { empty with atoms = any.atoms }
 end
 
 module Char = struct
   let has_char d c = Chars.contains c d.chars
   let is_empty d = Chars.is_empty d.chars
   let get d = d.chars
-  let any = { empty with hash=0; chars = any.chars }
+  let any = { empty with chars = any.chars }
 end
 
 let rec tuple = function
@@ -1919,11 +1747,10 @@ let rec_of_list o l =
     LabelMap.from_list (fun _ _ -> assert false)
       (List.map 
 	 (fun (opt,qname,typ) ->
-	    LabelPool.mk qname, 
-	    cons (if opt then Record.or_absent typ else typ))
+	    qname, cons (if opt then Record.or_absent typ else typ))
 	 l)
   in
-  record' (o,map)
+  record_fields (o,map)
 
 let empty_closed_record = rec_of_list false []
 let empty_open_record = rec_of_list true []
